@@ -16,21 +16,29 @@ enforce-consistent-style-for-gitlab.sh
 enforce-correct-shebangs.sh
 enforce-indent-two-spaces-outside-python.sh
 enforce-issue-number-for-todos.sh
-if [[ $0 != *.git/hooks/pre-commit ]]; then
-  # This one needs to connect to the linked websites and can potentially run long,
-  # let's skip it when this script is executed as git pre-commit hook.
+if [[ $CI ]] || command -v remark; then
   enforce-links-correct.sh
+else
+  echo 'Warning: remark CLI not installed, link check will be skipped. Use `npm install remark-cli remark-validate-links`'
 fi
 enforce-mocking-only-whitelisted-methods.sh
 enforce-newline-at-eof.sh
 enforce-release-notes-up-to-date.sh
 enforce-shell-scripts-pass-shellcheck.sh
+enforce-yq-check-for-each-y-yes-yq-check.sh
 prohibit-bash-usages-from-python.sh
+prohibit-current-date-in-tests.sh
 prohibit-deploy-step-in-circleci.sh
-prohibit-exempli-gratia-in-rst.sh
-prohibit-id-est-in-rst.sh
 prohibit-double-backticks-in-python.sh
+prohibit-exempli-gratia-in-rst.sh
+prohibit-fork-point-in-git-context.sh
+prohibit-github-in-gitlab-files.sh
+prohibit-github-mr-or-gitlab-pr.sh
+prohibit-gitlab-in-github-files.sh
+prohibit-id-est-in-rst.sh
 prohibit-markdown-links-in-rst.sh
+prohibit-mrs-in-github-files.sh
+prohibit-prs-in-gitlab-files.sh
 prohibit-single-backtick-in-rst.sh
 prohibit-split-backslash-n.sh
 prohibit-strings-split-without-delimiter.sh
