@@ -110,6 +110,10 @@ long_docs: Dict[str, str] = {
 
         Note that the current branch after the operation is still `develop`, just pointing to `feature/add-from-remote`'s tip now.
 
+        If the current branch `C` is annotated with `push=no` qualifier, the push is not performed.
+        If the downstream branch `D` is annotated with `slide-out=no` qualifier, the slide-out is not performed.
+        See help for `traverse` for more details on the qualifiers.
+
         <b>Options:</b>
            <b>-y</b>, <b>--yes</b>
               Don't ask for confirmation whether to fast-forward the current branch or whether to slide-out the downstream.
@@ -245,6 +249,20 @@ long_docs: Dict[str, str] = {
               Setting this config key to `true` will force `git machete github create-pr` to take PR description
               from the message body of the first unique commit of the branch, even if `.git/info/description` and/or `.github/pull_request_template.md` is present.
 
+           `machete.github.prDescriptionIntroStyle`:
+
+                 Select the style of the generated section ("intro") added to the PR description:
+
+                 * `full`                — include both a chain of upstream PRs (typically leading to `main`, `master`, `develop` etc.) and a tree of downstream PRs
+
+                 * `full-no-branches`    — same as `full`, but no branch names are included (only PR numbers & titles)
+
+                 * `up-only`             — default, include only a chain of upstream PRs
+
+                 * `up-only-no-branches` — same as `up-only`, but no branch names are included (only PR numbers & titles)
+
+                 * `none`                — prepend no intro to the PR description at all
+
            `machete.gitlab.{domain,remote,namespace,project}`:
 
                  GitLab self-managed domain
@@ -272,6 +290,20 @@ long_docs: Dict[str, str] = {
               Setting this config key to `true` will force `git machete gitlab create-mr` to take MR description
               from the message body of the first unique commit of the branch, even if `.git/info/description` and/or `.gitlab/merge_request_templates/Default.md` is present.
 
+           `machete.gitlab.mrDescriptionIntroStyle`:
+
+                 Select the style of the generated section ("intro") added to the MR description:
+
+                 * `full`                — include both a chain of upstream MRs (typically leading to `main`, `master`, `develop` etc.) and a tree of downstream MRs
+
+                 * `full-no-branches`    — same as `full`, but no branch names are included (only MR numbers & titles)
+
+                 * `up-only`             — default, include only a chain of upstream MRs
+
+                 * `up-only-no-branches` — same as `up-only`, but no branch names are included (only MR numbers & titles)
+
+                 * `none`                — prepend no intro to the MR description at all
+
            `machete.overrideForkPoint.<branch>.to`:
  
               Executing `git machete fork-point --override-to[-parent|-inferred|=<revision>] [<branch>]` sets up a fork point override for `<branch>`.
@@ -281,6 +313,25 @@ long_docs: Dict[str, str] = {
               There should be <b>no</b> need for the user to interact with this key directly,
               `git machete fork-point` with flags should be used instead.
 
+           `machete.squashMergeDetection`:
+ 
+              Controls the algorithm used to detect squash merges. Possible values are:
+
+                 * `none`: Fastest mode, with no squash merge/rebase detection. Only strict (fast-forward or 2-parent) merges are detected.
+
+                 * `simple` (default): Compares the tree (files & directories in the commit) of the downstream branch with the trees of the upstream branch.
+                   This detects squash merges/rebases as long as there exists a squash/rebase commit in the upstream that has the identical tree to what's in the downstream branch.
+
+                 * `exact`: Compares the patch (diff introduced by the commits) of the downstream branch with the patches of the upstream branch.
+                   This detects squash merges in more cases than `simple` mode.
+                   However, it might have a significant performance impact on large repositories as it requires computing patches for commits in the upstream branch.
+
+              This has an impact on:
+
+                 * whether a grey edge is displayed in `status`,
+
+                 * whether `traverse` suggests to slide out the branch.
+
            `machete.status.extraSpaceBeforeBranchName`:
  
               To make it easier to select branch name from the `status` output on certain terminals
@@ -303,6 +354,12 @@ long_docs: Dict[str, str] = {
               │
               └─ feature_branch2
 
+           `machete.traverse.fetch.<remote>`:
+              Configures the behavior of `git machete traverse` command for the given remote when `--fetch` flag is used.
+              If set to `false`, this remote will not be fetched before the traversal.
+              The default value of this configuration is `true`.
+              This is useful for excluding remotes that are temporarily offline, or take a long time to respond.
+
            `machete.traverse.push`:
  
               To change the behavior of `git machete traverse` command so that it doesn't push branches by default,
@@ -330,6 +387,9 @@ long_docs: Dict[str, str] = {
            `GITHUB_TOKEN`
               Used to store GitHub API token. Used by commands such as `anno --sync-github-prs` and `github`.
 
+           `GITLAB_TOKEN`
+              Used to store GitLab API token. Used by commands such as `anno --sync-gitlab-mrs` and `gitlab`.
+
    """,
     "delete-unmanaged": """
         <b>Usage:</b><b>
@@ -349,7 +409,7 @@ long_docs: Dict[str, str] = {
    """,
     "diff": """
         <b>Usage:</b><b>
-           git machete d[iff] [-s|--stat] [<branch>]</b>
+           git machete d[iff] [-s|--stat] [<branch>] [-- <pass-through-arguments>]</b>
 
         Runs `git diff` of the given branch tip against its fork point or, if none specified,
         of the current working tree against the fork point of the currently checked out branch.
@@ -358,8 +418,9 @@ long_docs: Dict[str, str] = {
         Note: the branch in question does not need to occur in the branch layout file.
 
         <b>Options:</b>
-           <b>-s</b>, <b>--stat</b>
-              Makes `git machete diff` pass `--stat` option to `git diff`, so that only summary (diffstat) is printed.
+
+        -- <pass-through-arguments>    Arguments to pass directly to the underlying `git diff`, for example `git machete diff -- --name-only`.
+        -s, --stat                     Makes `git machete diff` pass `--stat` option to `git diff`, so that only summary (diffstat) is printed.
    """,
     "discover": """
         <b>Usage:</b><b>
@@ -517,7 +578,7 @@ long_docs: Dict[str, str] = {
         <b>Usage:</b><b>
            git machete github <subcommand></b>
 
-        where `<subcommand>` is one of: `anno-prs`, `checkout-prs`, `create-pr`, `retarget-pr`.
+        where `<subcommand>` is one of: `anno-prs`, `checkout-prs`, `create-pr`, `retarget-pr`, `restack-pr` or `update-pr-descriptions`.
 
         Creates, checks out and manages GitHub PRs while keeping them reflected in branch layout file.
 
@@ -583,7 +644,7 @@ long_docs: Dict[str, str] = {
 
               `<PR-number-1> ... <PR-number-N>`    Pull request numbers to checkout.
 
-           `create-pr [--draft] [--title=<title>] [--yes]`:
+           `create-pr [--draft] [--title=<title>] [-U|--update-related-descriptions] [--yes]`:
  
               Creates a PR for the current branch, using the upstream (parent) branch as the PR base.
               Once the PR is successfully created, annotates the current branch with the new PR's number.
@@ -591,12 +652,14 @@ long_docs: Dict[str, str] = {
               If `.git/info/milestone` file is present, its contents (a single number — milestone id) are used as milestone.
               If `.git/info/reviewers` file is present, its contents (one GitHub login per line) are used to set reviewers.
 
-              The subject of the first unique commit of the branch is used as PR title.
-              If `.git/info/description` or `.github/pull_request_template.md` file is present, its contents are used as PR description.
+              Unless `--title` is provided, the subject of the first unique commit of the branch is used as PR title.
+              If `.git/info/description` or `.github/pull_request_template.md` template is present, its contents are used as PR description.
               Otherwise (or if `machete.github.forceDescriptionFromCommitMessage` is set), PR description is taken from message body of the first unique commit of the branch.
 
-              If the newly-created PR is stacked atop another PR, the actual PR description posted to GitHub will be prepended with a section
-              listing the entire related chain of PRs.
+              If the newly-created PR is stacked atop another PR, the actual PR description posted to GitHub will include a generated section ("intro")
+              listing the entire related chain of PRs. This section will be delimited with `<!-- start git-machete generated -->`
+              and `<!-- end git-machete generated -->` comments in Markdown. If a PR template file exists and contains these comments already,
+              the generated section will be placed between them; otherwise, it will be placed at the beginning.
 
               <b>Options:</b>
 
@@ -607,10 +670,15 @@ long_docs: Dict[str, str] = {
 
               Set the PR title explicitly (the default is to use the first included commit's message as the title).
 
+                 <b>-U</b>, <b>--update-related-descriptions</b>
+
+              Update the generated sections ("intros") of PR descriptions that list the upstream and/or downstream PRs.
+              See help for `git machete github update-pr-descriptions --related` for details.
+
                  <b>--yes</b>
               Do not ask for confirmation whether to push the branch.
 
-           `restack-pr`:
+           `restack-pr [-U|--update-related-descriptions]`:
  
               Perform the following sequence of actions:
 
@@ -625,14 +693,21 @@ long_docs: Dict[str, str] = {
               The drafting/undrafting is useful in case the GitHub repository has set up CODEOWNERS.
               Draft PRs don't get code owners automatically added as reviewers.
 
-           `retarget-pr [-b|--branch=<branch>] [--ignore-if-missing]`:
+              <b>Options:</b>
+
+                 <b>-U</b>, <b>--update-related-descriptions</b>
+
+              Update the generated sections ("intros") of PR descriptions that list the upstream and/or downstream PRs.
+              See help for `git machete github update-pr-descriptions --related` for details.
+
+           `retarget-pr [-b|--branch=<branch>] [--ignore-if-missing] [-U|--update-related-descriptions]`:
  
               Sets the base of the current (or specified) branch's PR to upstream (parent) branch, as seen by git machete (see `git machete show up`).
 
-              If after changing the base the PR ends up stacked atop another PR, the PR description posted to GitHub will be prepended with a section
-              listing the entire related chain of PRs.
+              If after changing the base the PR ends up stacked atop another PR, the PR description posted to GitHub will include
+              a generated section ("intro") listing the entire related chain of PRs.
 
-              This header will be updated or removed accordingly with the subsequent runs of `retarget-pr`.
+              This intro will be updated or removed accordingly with the subsequent runs of `retarget-pr`, even if the base branch is already up to date.
 
               <b>Options:</b>
 
@@ -644,6 +719,11 @@ long_docs: Dict[str, str] = {
 
               Ignore errors and quietly terminate execution if there is no PR opened for current (or specified) branch.
 
+                 <b>-U</b>, <b>--update-related-descriptions</b>
+
+              Update the generated sections ("intros") of PR descriptions that list the upstream and/or downstream PRs.
+              See help for `git machete github update-pr-descriptions --related` for details.
+
            `sync`:
  
               <b>Deprecated.</b> Use `github checkout-prs --mine`, `delete-unmanaged` and `slide-out --removed-from-remote`.
@@ -657,6 +737,28 @@ long_docs: Dict[str, str] = {
 
                  * deletes untracked managed branches that have no downstream branch.
 
+           `update-pr-descriptions [--all | --by=<github-login> | --mine | --related]`:
+ 
+              Updates the generated sections ("intros") of PR descriptions that list the upstream and/or downstream PRs
+              (depending on `machete.github.prDescriptionIntroStyle` git config key).
+
+              <b>Options:</b>
+
+                 <b>--all</b>
+              Update PR descriptions for all PRs in the repository.
+
+                 <b>--by=<github-login></b>
+
+              Update PR descriptions for all PRs authored by the given GitHub user, where `<github-login>` is the GitHub account name.
+
+                 <b>--mine</b>
+              Update PR descriptions for all PRs opened by the current user associated with the GitHub token.
+
+                 <b>--related</b>
+              Update PR descriptions for all PRs that are upstream and/or downstream of the PR for the current branch.
+              If `machete.github.prDescriptionIntroStyle` is `up-only` (default) or `up-only-no-branches`, then only downstream PR descriptions are updated.
+              If `machete.github.prDescriptionIntroStyle` is `full` or `full-no-branches`, then both downstream and upstream PR descriptions are updated.
+
         <b>Git config keys:</b>
            `machete.github.{domain,remote,organization,repository}` (all subcommands):
  
@@ -688,6 +790,20 @@ long_docs: Dict[str, str] = {
               Setting this config key to `true` will force `git machete github create-pr` to take PR description
               from the message body of the first unique commit of the branch, even if `.git/info/description` and/or `.github/pull_request_template.md` is present.
 
+           `machete.github.prDescriptionIntroStyle` (`create-pr`, `restack-pr` and `retarget-pr`):
+
+                 Select the style of the generated section ("intro") added to the PR description:
+
+                 * `full`                — include both a chain of upstream PRs (typically leading to `main`, `master`, `develop` etc.) and a tree of downstream PRs
+
+                 * `full-no-branches`    — same as `full`, but no branch names are included (only PR numbers & titles)
+
+                 * `up-only`             — default, include only a chain of upstream PRs
+
+                 * `up-only-no-branches` — same as `up-only`, but no branch names are included (only PR numbers & titles)
+
+                 * `none`                — prepend no intro to the PR description at all
+
         <b>Environment variables (all subcommands):</b>
            `GITHUB_TOKEN`
               GitHub API token.
@@ -697,11 +813,11 @@ long_docs: Dict[str, str] = {
         <b>Usage:</b><b>
            git machete gitlab <subcommand></b>
 
-        where `<subcommand>` is one of: `anno-mrs`, `checkout-mrs`, `create-mr`, `retarget-mr`.
+        where `<subcommand>` is one of: `anno-mrs`, `checkout-mrs`, `create-mr`, `retarget-mr`, `restack-mr` or `update-mr-descriptions`.
 
         Creates, checks out and manages GitLab MRs while keeping them reflected in branch layout file.
 
-        To allow GitLab API access for private repositories (and also to perform side-effecting actions like opening a PR,
+        To allow GitLab API access for private projects (and also to perform side-effecting actions like opening a MR,
         even in case of public projects), a GitLab API token with `api` scope is required, see https://gitlab.com/-/user_settings/personal_access_tokens.
         This will be resolved from the first of:
            * `GITLAB_TOKEN` env var,
@@ -738,9 +854,9 @@ long_docs: Dict[str, str] = {
 
            `checkout-mrs [--all | --by=<gitlab-login> | --mine | <MR-number-1> ... <MR-number-N>]`:
  
-              Check out the source branch of the given pull requests (specified by numbers or by a flag),
-              also traverse chain of pull requests upwards, adding branches one by one to git-machete and check them out locally.
-              Once the specified pull requests are checked out locally, annotate local branches with corresponding pull request numbers.
+              Check out the source branch of the given merge requests (specified by numbers or by a flag),
+              also traverse chain of merge requests upwards, adding branches one by one to git-machete and check them out locally.
+              Once the specified merge requests are checked out locally, annotate local branches with corresponding merge request numbers.
               If only one MR has been checked out, then switch the local repository's HEAD to its source branch.
 
               When the current user is NOT the author of the MR associated with that branch, adds `rebase=no push=no` branch qualifiers used by `git machete traverse`,
@@ -760,9 +876,9 @@ long_docs: Dict[str, str] = {
 
               <b>Parameters:</b>
 
-              `<MR-number-1> ... <MR-number-N>`    Pull request numbers to checkout.
+              `<MR-number-1> ... <MR-number-N>`    Merge request numbers to checkout.
 
-           `create-mr [--draft] [--title=<title>] [--yes]`:
+           `create-mr [--draft] [--title=<title>] [-U|--update-related-descriptions] [--yes]`:
  
               Creates a MR for the current branch, using the upstream (parent) branch as the MR source branch.
               Once the MR is successfully created, annotates the current branch with the new MR's number.
@@ -772,12 +888,14 @@ long_docs: Dict[str, str] = {
 
               If `.git/info/reviewers` file is present, its contents (one GitLab login per line) are used to set reviewers.
 
-              The subject of the first unique commit of the branch is used as MR title.
-              If `.git/info/description` or `.gitlab/merge_request_templates/Default.md` file is present, its contents are used as MR description.
+              Unless `--title` is provided, the subject of the first unique commit of the branch is used as MR title.
+              If `.git/info/description` or `.gitlab/merge_request_templates/Default.md` template is present, its contents are used as MR description.
               Otherwise (or if `machete.gitlab.forceDescriptionFromCommitMessage` is set), MR description is taken from message body of the first unique commit of the branch.
 
-              If the newly-created MR is stacked atop another MR, the actual MR description posted to GitLab will be prepended with a section
-              listing the entire related chain of MRs.
+              If the newly-created MR is stacked atop another MR, the actual MR description posted to GitLab will include a generated section ("intro")
+              listing the entire related chain of MRs. This section will be delimited with `<!-- start git-machete generated -->`
+              and `<!-- end git-machete generated -->` comments in Markdown. If an MR template file exists and contains these comments already,
+              the generated section will be placed between them; otherwise, it will be placed at the beginning.
 
               <b>Options:</b>
 
@@ -788,10 +906,15 @@ long_docs: Dict[str, str] = {
 
               Set the MR title explicitly (the default is to use the first included commit's message as the title).
 
+                 <b>-U</b>, <b>--update-related-descriptions</b>
+
+              Update the generated sections ("intros") of MR descriptions that list the upstream and/or downstream MRs.
+              See help for `git machete gitlab update-mr-descriptions --related` for details.
+
                  <b>--yes</b>
               Do not ask for confirmation whether to push the branch.
 
-           `restack-mr`:
+           `restack-mr [-U|--update-related-descriptions]`:
  
               Perform the following sequence of actions:
 
@@ -806,14 +929,21 @@ long_docs: Dict[str, str] = {
               The drafting/undrafting is useful in case the GitLab project has set up code owners.
               Draft MRs don't get code owners automatically added as reviewers.
 
-           `retarget-mr [-b|--branch=<branch>] [--ignore-if-missing]`:
+              <b>Options:</b>
+
+                 <b>-U</b>, <b>--update-related-descriptions</b>
+
+              Update the generated sections ("intros") of MR descriptions that list the upstream and/or downstream MRs.
+              See help for `git machete gitlab update-mr-descriptions --related` for details.
+
+           `retarget-mr [-b|--branch=<branch>] [--ignore-if-missing] [-U|--update-related-descriptions]`:
  
               Sets the target of the current (or specified) branch's MR to upstream (parent) branch, as seen by git machete (see `git machete show up`).
 
-              If after changing the target the MR ends up stacked atop another MR, the MR description posted to GitLab will be prepended with a section
-              listing the entire related chain of MRs.
+              If after changing the base the MR ends up stacked atop another MR, the MR description posted to GitLab will include
+              a generated section ("intro") listing the entire related chain of MRs.
 
-              This header will be updated or removed accordingly with the subsequent runs of `retarget-mr`.
+              This intro will be updated or removed accordingly with the subsequent runs of `retarget-mr`, even if the target branch is already up to date.
 
               <b>Options:</b>
 
@@ -825,6 +955,33 @@ long_docs: Dict[str, str] = {
 
               Ignore errors and quietly terminate execution if there is no MR opened for current (or specified) branch.
 
+                 <b>-U</b>, <b>--update-related-descriptions</b>
+
+              Update the generated sections ("intros") of MR descriptions that list the upstream and/or downstream MRs.
+              See help for `git machete gitlab update-mr-descriptions --related` for details.
+
+           `update-mr-descriptions [--all | --by=<gitlab-login> | --mine | --related]`:
+ 
+              Updates the generated sections ("intros") of MR descriptions that list the upstream and/or downstream MRs
+              (depending on `machete.gitlab.mrDescriptionIntroStyle` git config key).
+
+              <b>Options:</b>
+
+                 <b>--all</b>
+              Update MR descriptions for all MRs in the project.
+
+                 <b>--by=<gitlab-login></b>
+
+              Update MR descriptions for all MRs authored by the given GitLab user, where `<gitlab-login>` is the GitLab account name.
+
+                 <b>--mine</b>
+              Update MR descriptions for all MRs opened by the current user associated with the GitLab token.
+
+                 <b>--related</b>
+              Update MR descriptions for all MRs that are upstream and/or downstream of the MR for the current branch.
+              If `machete.gitlab.mrDescriptionIntroStyle` is `up-only` (default) or `up-only-no-branches`, then only downstream MR descriptions are updated.
+              If `machete.gitlab.mrDescriptionIntroStyle` is `full` or `full-no-branches`, then both downstream and upstream MR descriptions are updated.
+
         <b>Git config keys:</b>
            `machete.gitlab.{domain,remote,namespace,project}` (all subcommands):
  
@@ -856,6 +1013,20 @@ long_docs: Dict[str, str] = {
               Setting this config key to `true` will force `git machete gitlab create-mr` to take MR description
               from the message body of the first unique commit of the branch, even if `.git/info/description` and/or `.gitlab/merge_request_templates/Default.md` is present.
 
+           `machete.gitlab.mrDescriptionIntroStyle` (`create-mr`, `restack-mr` and `retarget-mr`):
+
+                 Select the style of the generated section ("intro") added to the MR description:
+
+                 * `full`                — include both a chain of upstream MRs (typically leading to `main`, `master`, `develop` etc.) and a tree of downstream MRs
+
+                 * `full-no-branches`    — same as `full`, but no branch names are included (only MR numbers & titles)
+
+                 * `up-only`             — default, include only a chain of upstream MRs
+
+                 * `up-only-no-branches` — same as `up-only`, but no branch names are included (only MR numbers & titles)
+
+                 * `none`                — prepend no intro to the MR description at all
+
         <b>Environment variables (all subcommands):</b>
            `GITLAB_TOKEN`
               GitLab API token.
@@ -994,12 +1165,16 @@ long_docs: Dict[str, str] = {
    """,
     "log": """
         <b>Usage:</b><b>
-           git machete l[og] [<branch>]</b>
+           git machete l[og] [<branch>] [-- <pass-through-arguments>]</b>
 
         Runs `git log` for the range of commits from tip of the given branch (or current branch, if none specified) back to its fork point.
         See help for `fork-point` for more details on meaning of the fork point.
 
         Note: the branch in question does not need to occur in the branch layout file.
+
+        <b>Options:</b>
+
+        -- <pass-through-arguments>    Arguments to pass directly to the underlying `git log`, for example `git machete log -- --patch`.
    """,
     "reapply": """
         <b>Usage:</b><b>
@@ -1049,11 +1224,14 @@ long_docs: Dict[str, str] = {
     "slide-out": """
         <b>Usage:</b><b>
            git machete slide-out --removed-from-remote [--delete]
-           git machete slide-out [-d|--down-fork-point=<down-fork-point-commit>] [--delete] [-M|--merge] [-n|--no-edit-merge|--no-interactive-rebase] [<branch> [<branch> [<branch> ...]]]</b>
+           git machete slide-out [-d|--down-fork-point=<down-fork-point-commit>] [--delete]
+                                 [-M|--merge] [-n|--no-edit-merge|--no-interactive-rebase]
+                                 [<branch> [<branch> [<branch> ...]]]</b>
 
         Removes the given branch (or multiple branches) from the branch layout.
         If no branch has been specified, current branch is slid out.
-        If `--removed-from-remote` is specified, all branches that have been removed from the remote are slid out instead.
+        If `--removed-from-remote` is specified, all branches that have been removed from the remote
+        and do <b>not</b> have a child branch are slid out instead.
 
         Also, if the last branch in the specified chain of `[<branch> [<branch>]]` had any children,
         these children are synced to the parent of the first specified branch.
@@ -1083,10 +1261,13 @@ long_docs: Dict[str, str] = {
 
         The provided branches must form a chain — all of the following conditions must be met:
            * for i=1..N-1, (i+1)-th branch must be the only downstream (child) branch of the i-th branch,
-           * all provided branches must have an upstream branch (so, in other words, roots of branch dependency tree cannot be slid out).
+           * all provided branches must have an upstream branch (so, in other words, roots of branch layout cannot be slid out).
 
         Note: Unless `--delete` is passed, `slide-out` doesn't delete any branches from git, just removes them from the tree of branch dependencies.
 
+        Note: if a child branch is annotated with `rebase=no` qualifier, the rebase is not performed.
+        See help for `traverse` for more details on the qualifiers.
+
         <b>Options:</b>
            <b>-d</b>, <b>--down-fork-point=<down-fork-point-commit></b>
               If updating by rebase, specifies the alternative fork point for downstream branches for the operation.
@@ -1148,7 +1329,9 @@ long_docs: Dict[str, str] = {
    """,
     "status": """
         <b>Usage:</b><b>
-           git machete s[tatus] [--color=WHEN] [-l|--list-commits] [-L|--list-commits-with-hashes] [--no-detect-squash-merges]</b>
+           git machete s[tatus] [--color=WHEN]
+                                [-l|--list-commits] [-L|--list-commits-with-hashes]
+                                [--squash-merge-detection=MODE]</b>
 
         Displays a tree-shaped status of the branches listed in the branch layout file.
 
@@ -1222,10 +1405,34 @@ long_docs: Dict[str, str] = {
            <b>-L</b>, <b>--list-commits-with-hashes</b>
               Additionally list the short hashes and messages of commits introduced on each branch.
            <b>--no-detect-squash-merges</b>
+              <b>Deprecated</b>, use `--squash-merge-detection=none` instead.
               Only consider strict (fast-forward or 2-parent) merges, rather than rebase/squash merges,
               when detecting if a branch is merged into its upstream (parent).
+           <b>--squash-merge-detection=MODE</b>
+              Specify the mode for detection of rebase/squash merges (grey edges).
+              `MODE` can be `none` (fastest, no squash merges are detected), `simple` (default) or `exact` (slowest).
+              See the below paragraph on `machete.squashMergeDetection` git config key for more details.
 
         <b>Git config keys:</b>
+           `machete.squashMergeDetection`:
+ 
+              Controls the algorithm used to detect squash merges. Possible values are:
+
+                 * `none`: Fastest mode, with no squash merge/rebase detection. Only strict (fast-forward or 2-parent) merges are detected.
+
+                 * `simple` (default): Compares the tree (files & directories in the commit) of the downstream branch with the trees of the upstream branch.
+                   This detects squash merges/rebases as long as there exists a squash/rebase commit in the upstream that has the identical tree to what's in the downstream branch.
+
+                 * `exact`: Compares the patch (diff introduced by the commits) of the downstream branch with the patches of the upstream branch.
+                   This detects squash merges in more cases than `simple` mode.
+                   However, it might have a significant performance impact on large repositories as it requires computing patches for commits in the upstream branch.
+
+              This has an impact on:
+
+                 * whether a grey edge is displayed in `status`,
+
+                 * whether `traverse` suggests to slide out the branch.
+
            `machete.status.extraSpaceBeforeBranchName`
  
               To make it easier to select branch name from the `status` output on certain terminals
@@ -1252,9 +1459,10 @@ long_docs: Dict[str, str] = {
     "traverse": """
         <b>Usage:</b><b>
            git machete t[raverse] [-F|--fetch] [-l|--list-commits] [-M|--merge]
-                                  [-n|--no-edit-merge|--no-interactive-rebase] [--no-detect-squash-merges]
-                                  [--[no-]push] [--[no-]push-untracked]
-                                  [--return-to=WHERE] [--start-from=WHERE] [-w|--whole] [-W] [-y|--yes]</b>
+                                  [-n|--no-edit-merge|--no-interactive-rebase] [--[no-]push] [--[no-]push-untracked]
+                                  [--return-to=WHERE] [--start-from=WHERE] [--squash-merge-detection=MODE]
+                                  [-H|--sync-github-prs|-L|--sync-gitlab-mrs]
+                                  [-w|--whole] [-W] [-y|--yes]</b>
 
         Traverses the branches in the order as they occur in branch layout file.
         By default, `traverse` starts from the current branch.
@@ -1274,6 +1482,11 @@ long_docs: Dict[str, str] = {
               - asks the user whether to <b>reset</b> (`git reset --keep`) the branch to its remote counterpart
            * otherwise, if the branch is behind its remote counterpart:
               - asks the user whether to <b>pull</b> the branch;
+           * if `-H`/`--sync-github-prs` or `-L`/`--sync-gitlab-mrs` option is present:
+              - asks the user whether to <b>create</b> a PR/MR for the given branch if it's untracked,
+              - asks the user whether to <b>retarget</b> the PR/MR if it exists for the given branch,
+                and its base/target branch in GitHub/GitLab is different than the upstream in machete file
+                (just as `git machete github retarget-pr` and `git machete gitlab retarget-mr` would do);
            * and finally, if any of the above operations has been successfully completed:
               - prints the updated `status`.
 
@@ -1284,7 +1497,7 @@ long_docs: Dict[str, str] = {
         If the traverse flow is stopped (typically due to merge/rebase conflicts), just run `git machete traverse` after the merge/rebase is finished.
         It will pick up the walk from the current branch.
         Unlike with `git rebase` or `git cherry-pick`, there is no special `--continue` flag, as `traverse` is stateless.
-        `traverse` doesn't keep a state of its own like `git rebase` does in `.git/rebase-apply/`.
+        `traverse` does <b>not</b> keep a state of its own like `git rebase` does in `.git/rebase-apply/`.
 
         The rebase, push and slide-out behaviors of `traverse` can also be customized for each branch separately using branch qualifiers.
         There are `push=no`, `rebase=no` and `slide-out=no` qualifiers that can be used to opt out of default behavior (rebasing, pushing and sliding the branch out).
@@ -1306,6 +1519,10 @@ long_docs: Dict[str, str] = {
         <b>Options:</b>
            <b>-F</b>, <b>--fetch</b>
               Fetch the remotes of all managed branches at the beginning of traversal (no `git pull` involved, only `git fetch`).
+           <b>-H</b>, <b>--sync-github-prs</b>
+              Retarget the PR if it exists for the given branch and its base branch in GitHub is different than the upstream in machete file
+           <b>-L</b>, <b>--sync-gitlab-mrs</b>
+              Retarget the MR if it exists for the given branch and its target branch in GitLab is different than the upstream in machete file
            <b>-l</b>, <b>--list-commits</b>
               When printing the status, additionally list the messages of commits introduced on each branch.
            <b>-M</b>, <b>--merge</b>
@@ -1313,6 +1530,7 @@ long_docs: Dict[str, str] = {
            <b>-n</b>
               If updating by rebase, equivalent to `--no-interactive-rebase`. If updating by merge, equivalent to `--no-edit-merge`.
            <b>--no-detect-squash-merges</b>
+              <b>Deprecated</b>, use `--squash-merge-detection=none` instead.
               Only consider strict (fast-forward or 2-parent) merges, rather than rebase/squash merges,
               when detecting if a branch is merged into its upstream (parent).
            <b>--no-edit-merge</b>
@@ -1337,6 +1555,10 @@ long_docs: Dict[str, str] = {
               (nearest remaining branch in case the `here` branch has been slid out by the traversal) or
               `stay` (the default — just stay wherever the traversal stops). Note: when user quits by `q`/`yq`
               or when traversal is stopped because one of git actions fails, the behavior is always `stay`.
+           <b>--squash-merge-detection=MODE</b>
+              Specifies the mode for detection of rebase/squash merges (grey edges).
+              `MODE` can be `none` (fastest, no squash merges are detected), `simple` (default) or `exact` (slowest).
+              See the below paragraph on `machete.squashMergeDetection` git config key for more details.
            <b>--start-from=WHERE</b>
               Specifies the branch to start the traversal from; WHERE can be `here`
               (the default — current branch, must be managed by git machete), `root` (root branch of the current branch,
@@ -1356,6 +1578,31 @@ long_docs: Dict[str, str] = {
               Example: `GIT_MACHETE_REBASE_OPTS="--keep-empty --rebase-merges" git machete traverse`.
 
         <b>Git config keys:</b>
+           `machete.squashMergeDetection`:
+ 
+              Controls the algorithm used to detect squash merges. Possible values are:
+
+                 * `none`: Fastest mode, with no squash merge/rebase detection. Only strict (fast-forward or 2-parent) merges are detected.
+
+                 * `simple` (default): Compares the tree (files & directories in the commit) of the downstream branch with the trees of the upstream branch.
+                   This detects squash merges/rebases as long as there exists a squash/rebase commit in the upstream that has the identical tree to what's in the downstream branch.
+
+                 * `exact`: Compares the patch (diff introduced by the commits) of the downstream branch with the patches of the upstream branch.
+                   This detects squash merges in more cases than `simple` mode.
+                   However, it might have a significant performance impact on large repositories as it requires computing patches for commits in the upstream branch.
+
+              This has an impact on:
+
+                 * whether a grey edge is displayed in `status`,
+
+                 * whether `traverse` suggests to slide out the branch.
+
+           `machete.traverse.fetch.<remote>`:
+              Configures the behavior of `git machete traverse` command for the given remote when `--fetch` flag is used.
+              If set to `false`, this remote will not be fetched before the traversal.
+              The default value of this configuration is `true`.
+              This is useful for excluding remotes that are temporarily offline, or take a long time to respond.
+
            `machete.traverse.push`
  
               To change the behavior of `git machete traverse` command so that it doesn't push branches by default,
@@ -1366,7 +1613,7 @@ long_docs: Dict[str, str] = {
    """,
     "update": """
         <b>Usage:</b><b>
-           git machete update [-f|--fork-point=] [-M|--merge] [-n|--no-edit-merge|--no-interactive-rebase]</b>
+           git machete update [-f|--fork-point=<fork-point-commit>] [-M|--merge] [-n|--no-edit-merge|--no-interactive-rebase]</b>
 
         Synchronizes the current branch with its upstream (parent) branch either by rebase (default) or by merge (if `--merge` option passed).
 
