@@ -10,10 +10,9 @@ from pathlib import Path
 from typing import Any, Dict, List, NamedTuple, Optional, Tuple
 
 from .code_hosting import (CodeHostingClient, CodeHostingGitConfigKeys,
-                           CodeHostingSpec, OrganizationAndRepositoryAndGitUrl,
-                           PullRequest)
-from .exceptions import (MacheteException, UnexpectedMacheteException,
-                         UnprocessableEntityOrConflictHTTPError)
+                           CodeHostingSpec, OrganizationAndRepository,
+                           OrganizationAndRepositoryAndGitUrl, PullRequest)
+from .exceptions import MacheteException, UnexpectedMacheteException
 from .git_operations import LocalBranchShortName
 from .utils import bold, compact_dict, debug, popen_cmd, warn
 
@@ -150,40 +149,8 @@ class GitHubClient(CodeHostingClient):
     # As of Dec 2022, GitHub API never returns more than 100 PRs, even if per_page query param is above 100.
     MAX_PULLS_PER_PAGE_COUNT = 100
 
-    @classmethod
-    def spec(cls) -> CodeHostingSpec:
-        return CodeHostingSpec(
-            base_branch_name='base',
-            client_class=cls,
-            default_domain=cls.DEFAULT_GITHUB_DOMAIN,
-            display_name='GitHub',
-            git_machete_command='github',
-            head_branch_name='head',
-            organization_name='organization',
-            # https://docs.github.com/en/communities/using-templates-to-encourage-useful-issues-and-pull-requests/creating-a-pull-request-template-for-your-repository
-            pr_description_path=['.github', 'pull_request_template.md'],
-            pr_ordinal_char='#',
-            pr_full_name='pull request',
-            pr_short_name='PR',
-            repository_name='repository',
-            token_providers_message=(
-                f'\n\t1. `{GITHUB_TOKEN_ENV_VAR}` environment variable\n'
-                '\t2. Content of the `~/.github-token` file\n'
-                '\t3. Current auth token from the `gh` GitHub CLI\n'
-                '\t4. Current auth token from the `hub` GitHub CLI\n'
-            ),
-            git_config_keys=CodeHostingGitConfigKeys(
-                domain='machete.github.domain',
-                organization='machete.github.organization',
-                repository='machete.github.repository',
-                remote='machete.github.remote',
-                annotate_with_urls='machete.github.annotateWithUrls',
-                force_description_from_commit_message='machete.github.forceDescriptionFromCommitMessage',
-            )
-        )
-
-    def __init__(self, spec: CodeHostingSpec, domain: str, organization: str, repository: str) -> None:
-        super().__init__(spec, domain, organization, repository)
+    def __init__(self, *, domain: str, organization: str, repository: str) -> None:
+        super().__init__(domain, organization, repository)
         self.__token: Optional[GitHubToken] = GitHubToken.for_domain(domain)
 
     def __get_pull_request_from_json(self, pr_json: Dict[str, Any]) -> "PullRequest":
@@ -196,9 +163,10 @@ class GitHubClient(CodeHostingClient):
             head_repo_id=pr_json['head']['repo']['id'],
             html_url=pr_json['html_url'],
             state=pr_json['state'],
+            title=pr_json['title'],
             description=pr_json['body'])
 
-    def __fire_github_api_request(self, method: str, path: str, request_body: Optional[Dict[str, Any]] = None) -> Any:
+    def __fire_github_api_request(self, method: str, path: str, request_body: Optional[Dict[str, Any]] = None) -> Any:  # noqa: KW
         headers: Dict[str, str] = {
             'Content-type': 'application/json',
             'User-Agent': 'git-machete',
@@ -221,7 +189,7 @@ class GitHubClient(CodeHostingClient):
               f'bearer token and request body {compact_dict(request_body) if request_body else "<none>"}')
 
         try:
-            with urllib.request.urlopen(http_request) as response:
+            with urllib.request.urlopen(http_request, context=self.ssl_context) as response:
                 parsed_response_body: Any = json.loads(response.read().decode())
                 # https://docs.github.com/en/rest/guides/using-pagination-in-the-rest-api?apiVersion=2022-11-28#using-link-headers
                 link_header: str = response.info()["link"]
@@ -239,14 +207,22 @@ class GitHubClient(CodeHostingClient):
             if err.code == http.HTTPStatus.UNPROCESSABLE_ENTITY:
                 error_response = json.loads(err.read().decode())
                 error_reason: str = self.__extract_failure_info_from_422(error_response)
-                raise UnprocessableEntityOrConflictHTTPError(error_reason)
+                if 'A pull request already exists for' in error_reason:
+                    raise MacheteException(error_reason)
+                elif 'Reviews may only be requested from collaborators.' in error_reason:
+                    print()
+                    warn(f"There are some invalid reviewers (non-collaborators) in .git{os.path.sep}info{os.path.sep}reviewers file.\n"
+                         "Skipped adding reviewers to the pull request.")
+                else:
+                    raise UnexpectedMacheteException(
+                        f'GitHub API returned 422 (Unprocessable Entity) HTTP status with error message: `{error_reason}`.')
             elif err.code in (http.HTTPStatus.UNAUTHORIZED, http.HTTPStatus.FORBIDDEN):
                 first_line = f'GitHub API returned `{err.code}` HTTP status with error message: `{err.reason}`\n'
                 last_line = 'You can also use a different token provider - see `git machete help github` for details.'
                 if self.__token:
                     raise MacheteException(
                         first_line + f'Make sure that the GitHub API token provided by {self.__token.provider} '
-                                     f'is valid and allows for access to `{method.upper()}` `{url_prefix}{path}`.\n' + last_line)
+                        f'is valid and allows for access to `{method.upper()}` `{url_prefix}{path}`.\n' + last_line)
                 else:
                     raise MacheteException(
                         first_line + 'You might not have the required permissions for this repository.\n'
@@ -256,7 +232,7 @@ class GitHubClient(CodeHostingClient):
                 # TODO (#164): make a dedicated exception here
                 raise MacheteException(
                     f'`{method} {url}` request ended up in 404 response from GitHub. A valid GitHub API token is required.\n'
-                    f'Provide a GitHub API token with `repo` access via one of the: {self._spec.token_providers_message} '
+                    f'Provide a GitHub API token with `repo` access via one of the: {GITHUB_CLIENT_SPEC.token_providers_message} '
                     f'Visit `https://{self.domain}/settings/tokens` to generate a new one.')
             # See https://stackoverflow.com/a/62385184 for why 307 for POST/PATCH isn't automatically followed by urllib,
             # unlike 307 for GET, or 301/302 for all HTTP methods.
@@ -288,12 +264,20 @@ class GitHubClient(CodeHostingClient):
                      f'New organization is {bold(new_org)} and new repository is {bold(new_repo)}.\n'
                      'You can update your remote repository via: `git remote set-url <remote_name> <new_repository_url>`.')
                 return result
+            elif err.code >= 500:
+                raise MacheteException(f'GitHub API returned `{err.code}` '
+                                       f'HTTP status with error message: `{err.reason}`.')  # pragma: no cover
             else:
                 raise UnexpectedMacheteException(f'GitHub API returned `{err.code}` HTTP status with error message: `{err.reason}`.')
         except OSError as e:  # pragma: no cover
             raise MacheteException(f'Could not connect to {url_prefix}: {e}')
 
-    def __fire_github_api_repo_request(self, method: str, path_suffix: str, request_body: Optional[Dict[str, Any]] = None) -> Any:
+    def __fire_github_api_repo_request(  # noqa: KW
+            self,
+            method: str,
+            path_suffix: str,
+            request_body: Optional[Dict[str, Any]] = None
+    ) -> Any:
         path = f'/repos/{self.organization}/{self.repository}{path_suffix}'
         return self.__fire_github_api_request(method=method, path=path, request_body=request_body)
 
@@ -316,9 +300,11 @@ class GitHubClient(CodeHostingClient):
         else:
             return str(response)
 
-    def create_pull_request(self, head: str, base: str, title: str, description: str, draft: bool) -> PullRequest:
+    def create_pull_request(self, head: str, head_org_repo: OrganizationAndRepository, *,
+                            base: str, title: str, description: str, draft: bool) -> PullRequest:
         request_body: Dict[str, Any] = {
             'head': head,
+            'head_repo': f'{head_org_repo.organization}/{head_org_repo.repository}',
             'base': base,
             'title': title,
             'body': description,
@@ -358,7 +344,7 @@ class GitHubClient(CodeHostingClient):
     # See https://docs.github.com/en/rest/pulls/pulls?apiVersion=2022-11-28#update-a-pull-request
     # and https://github.com/orgs/community/discussions/45174.
     # GraphQL (v4) API mutation needs to be used for that purpose.
-    def set_draft_status_of_pull_request(self, number: int, target_draft_status: bool) -> bool:
+    def set_draft_status_of_pull_request(self, number: int, *, target_draft_status: bool) -> bool:
         """Returns true if PR had a different draft status, and draft status has been toggled.
         Returns false if PR already had the desired draft status, and hence draft status has NOT been toggled."""
 
@@ -434,3 +420,38 @@ class GitHubClient(CodeHostingClient):
     def get_ref_name_for_pull_request(self, pr_number: int) -> str:
         # See `git ls-remote` for any GitHub remote.
         return f"pull/{pr_number}/head"
+
+
+GITHUB_CLIENT_SPEC = CodeHostingSpec(
+    base_branch_name='base',
+    client_class=GitHubClient,
+    default_domain=GitHubClient.DEFAULT_GITHUB_DOMAIN,
+    display_name='GitHub',
+    git_machete_command='github',
+    head_branch_name='head',
+    organization_name='organization',
+    # https://docs.github.com/en/communities/using-templates-to-encourage-useful-issues-and-pull-requests/creating-a-pull-request-template-for-your-repository
+    pr_description_path=['.github', 'pull_request_template.md'],
+    pr_full_name='pull request',
+    pr_intro_br_before_branches=False,
+    pr_intro_explicit_title=False,
+    pr_ordinal_char='#',
+    pr_short_name='PR',
+    pr_short_name_article='a',
+    repository_name='repository',
+    token_providers_message=(
+        f'\n\t1. `{GITHUB_TOKEN_ENV_VAR}` environment variable\n'
+        '\t2. Content of the `~/.github-token` file\n'
+        '\t3. Current auth token from the `gh` GitHub CLI\n'
+        '\t4. Current auth token from the `hub` GitHub CLI\n'
+    ),
+    git_config_keys=CodeHostingGitConfigKeys(
+        domain='machete.github.domain',
+        organization='machete.github.organization',
+        repository='machete.github.repository',
+        remote='machete.github.remote',
+        annotate_with_urls='machete.github.annotateWithUrls',
+        force_description_from_commit_message='machete.github.forceDescriptionFromCommitMessage',
+        pr_description_intro_style='machete.github.prDescriptionIntroStyle',
+    )
+)
