@@ -12,11 +12,10 @@ from typing import Any, Dict, List, NamedTuple, Optional
 from git_machete.code_hosting import (CodeHostingClient,
                                       CodeHostingGitConfigKeys,
                                       CodeHostingSpec,
+                                      OrganizationAndRepository,
                                       OrganizationAndRepositoryAndGitUrl,
                                       PullRequest)
-from git_machete.exceptions import (MacheteException,
-                                    UnexpectedMacheteException,
-                                    UnprocessableEntityOrConflictHTTPError)
+from git_machete.exceptions import MacheteException, UnexpectedMacheteException
 from git_machete.git_operations import LocalBranchShortName
 from git_machete.utils import compact_dict, debug, map_truthy_only, popen_cmd
 
@@ -98,46 +97,12 @@ class GitLabClient(CodeHostingClient):
     DEFAULT_GITLAB_DOMAIN = "gitlab.com"
     MAX_PULLS_PER_PAGE_COUNT = 100
 
-    def __init__(self, spec: CodeHostingSpec, domain: str, organization: str, repository: str) -> None:
-        super().__init__(spec, domain, organization, repository)
+    def __init__(self, *, domain: str, organization: str, repository: str) -> None:
+        super().__init__(domain, organization, repository)
         self.__token: Optional[GitLabToken] = GitLabToken.for_domain(domain)
 
-    @classmethod
-    def spec(cls) -> CodeHostingSpec:
-        return CodeHostingSpec(
-            base_branch_name='target',
-            client_class=cls,
-            default_domain=cls.DEFAULT_GITLAB_DOMAIN,
-            display_name='GitLab',
-            git_machete_command='gitlab',
-            head_branch_name='source',
-            organization_name='namespace',
-            # https://docs.gitlab.com/ee/user/project/description_templates.html#create-a-merge-request-template
-            # https://docs.gitlab.com/ee/user/project/description_templates.html#set-a-default-template-for-merge-requests-and-issues
-            # Actual MR template resolution for GitLab has a complex hierarchy of templates, including
-            # project-level, group-level and instance-level templates - and also the ability to "choose" a template.
-            # To keep things simple, we'll only support the "Default.md" template for now.
-            pr_description_path=['.gitlab', 'merge_request_templates', 'Default.md'],
-            pr_full_name='merge request',
-            pr_ordinal_char='!',
-            pr_short_name='MR',
-            repository_name='project',
-            token_providers_message=(
-                f'\n\t1. `{GITLAB_TOKEN_ENV_VAR}` environment variable\n'
-                '\t2. Content of the `~/.gitlab-token` file\n'
-                '\t3. Current auth token from the `glab` GitLab CLI\n'
-            ),
-            git_config_keys=CodeHostingGitConfigKeys(
-                domain='machete.gitlab.domain',
-                organization='machete.gitlab.namespace',
-                repository='machete.gitlab.project',
-                remote='machete.gitlab.remote',
-                annotate_with_urls='machete.gitlab.annotateWithUrls',
-                force_description_from_commit_message='machete.gitlab.forceDescriptionFromCommitMessage',
-            )
-        )
-
-    def __get_merge_request_from_json(self, mr_json: Dict[str, Any]) -> PullRequest:
+    @staticmethod
+    def __get_merge_request_from_json(mr_json: Dict[str, Any]) -> PullRequest:
         return PullRequest(
             number=int(mr_json['iid']),
             display_prefix='MR !',
@@ -147,9 +112,10 @@ class GitLabClient(CodeHostingClient):
             head_repo_id=int(mr_json['source_project_id']),
             html_url=mr_json['web_url'],
             state=mr_json['state'],
+            title=mr_json['title'],
             description=mr_json['description'])
 
-    def __fire_gitlab_api_request(self, method: str, path: str, request_body: Optional[Dict[str, Any]] = None) -> Any:
+    def __fire_gitlab_api_request(self, method: str, path: str, request_body: Optional[Dict[str, Any]] = None) -> Any:  # noqa: KW
         headers: Dict[str, str] = {
             "Content-Type": "application/json"
         }
@@ -164,7 +130,7 @@ class GitLabClient(CodeHostingClient):
               f'bearer token and request body {compact_dict(request_body) if request_body else "<none>"}')
 
         try:
-            with urllib.request.urlopen(http_request) as response:
+            with urllib.request.urlopen(http_request, context=self.ssl_context) as response:
                 parsed_response_body: Any = json.loads(response.read().decode())
                 # https://docs.gitlab.com/ee/api/rest/#pagination-link-header
                 link_header: str = response.info()["link"]
@@ -182,14 +148,18 @@ class GitLabClient(CodeHostingClient):
             if err.code == http.HTTPStatus.CONFLICT:
                 error_response = json.loads(err.read().decode())
                 error_reason: str = self.__extract_failure_info_from_409(error_response)
-                raise UnprocessableEntityOrConflictHTTPError(error_reason)
+                if 'Another open merge request already exists for this source branch:' in error_reason:
+                    raise MacheteException(error_reason)
+                else:
+                    raise UnexpectedMacheteException(
+                        f'GitLab API returned 409 (Conflict) HTTP status with error message: `{error_reason}`.')
             elif err.code in (http.HTTPStatus.UNAUTHORIZED, http.HTTPStatus.FORBIDDEN):
                 first_line = f'GitLab API returned `{err.code}` HTTP status with error message: `{err.reason}`\n'
                 last_line = 'You can also use a different token provider - see `git machete help gitlab` for details.'
                 if self.__token:
                     raise MacheteException(
                         first_line + f'Make sure that the GitLab API token provided by {self.__token.provider} '
-                                     f'is valid and allows for access to `{method.upper()}` `{url_prefix}{path}`.\n' + last_line)
+                        f'is valid and allows for access to `{method.upper()}` `{url_prefix}{path}`.\n' + last_line)
                 else:
                     raise MacheteException(
                         first_line + 'You might not have the required permissions for this project.\n'
@@ -200,7 +170,7 @@ class GitLabClient(CodeHostingClient):
                 # TODO (#164): make a dedicated exception here
                 raise MacheteException(
                     f'`{method} {url}` request ended up in 404 response from GitLab. A valid GitLab API token is required.\n'
-                    f'Provide a GitLab API token with `api` access via one of the: {self._spec.token_providers_message} '
+                    f'Provide a GitLab API token with `api` access via one of the: {GITLAB_CLIENT_SPEC.token_providers_message} '
                     f'Visit `https://{self.domain}/-/user_settings/personal_access_tokens` to generate a new one.')
             elif err.code == http.HTTPStatus.METHOD_NOT_ALLOWED:
                 error_response = json.loads(err.read().decode())
@@ -210,18 +180,30 @@ class GitLabClient(CodeHostingClient):
                         f"ended up in `{error_response.get('message')}` response from GitLab.\n"
                         "Please report this error as a comment under `https://github.com/VirtusLab/git-machete/issues/1212`.\n"
                         "As a workaround for now, please check `git remote -v`.\n"
-                        "Most likely you use an old URL of a repository that has been moved since.\n"
+                        "Most likely you use an old URL of a project that has been moved since.\n"
                         "Use `git remote set-url <remote> <URL>` to update the URL.")
                 else:
                     UnexpectedMacheteException(
                         f'GitLab API returned 405 (Method Not Allowed) HTTP status with error message: `{err.reason}`.')
+            elif err.code >= 500:
+                raise MacheteException(f'GitLab API returned `{err.code}` '
+                                       f'HTTP status with error message: `{err.reason}`.')  # pragma: no cover
             else:
                 raise UnexpectedMacheteException(f'GitLab API returned `{err.code}` HTTP status with error message: `{err.reason}`.')
         except OSError as e:  # pragma: no cover
             raise MacheteException(f'Could not connect to {url_prefix}: {e}')
 
-    def __fire_gitlab_api_project_request(self, method: str, path_suffix: str, request_body: Optional[Dict[str, Any]] = None) -> Any:
-        project = urllib.parse.quote(f"{self.organization}/{self.repository}", safe='')  # `safe` empty, so that `/` is encoded as well
+    @staticmethod
+    def __url_encode_project_name(organization: str, repository: str) -> str:  # noqa: KW
+        return urllib.parse.quote(f"{organization}/{repository}", safe='')  # `safe` empty, so that `/` is encoded as well
+
+    def __fire_gitlab_api_project_request(  # noqa: KW
+            self,
+            method: str,
+            path_suffix: str,
+            request_body: Optional[Dict[str, Any]] = None
+    ) -> Any:
+        project = self.__url_encode_project_name(self.organization, self.repository)
         path = f'/projects/{project}{path_suffix}'
         return self.__fire_gitlab_api_request(method=method, path=path, request_body=request_body)
 
@@ -235,14 +217,25 @@ class GitLabClient(CodeHostingClient):
         else:
             return str(response)
 
-    def create_pull_request(self, head: str, base: str, title: str, description: str, draft: bool) -> PullRequest:
+    def __get_repo_id_by_org_repo(self) -> int:
+        project = self.__url_encode_project_name(self.organization, self.repository)
+        return int(self.__fire_gitlab_api_request(method='GET', path=f'/projects/{project}')['id'])  # cast to int to satisfy mypy
+
+    def create_pull_request(self, head: str, head_org_repo: OrganizationAndRepository,
+                            *, base: str, title: str, description: str, draft: bool) -> PullRequest:
+        # The GitLab API for creating MRs across projects is somewhat confusing:
+        # URL path needs to point to the source project, but the MR is still created in the target project,
+        # which needs to pointed to by `target_project_id` JSON property.
         request_body: Dict[str, Any] = {
             'source_branch': head,
             'target_branch': base,
+            'target_project_id': self.__get_repo_id_by_org_repo(),
             'title': ('Draft: ' if draft else '') + title,
             'description': description,
         }
-        mr = self.__fire_gitlab_api_project_request(method='POST', path_suffix='/merge_requests', request_body=request_body)
+        head_project = self.__url_encode_project_name(head_org_repo.organization, head_org_repo.repository)
+        path = f'/projects/{head_project}/merge_requests'
+        mr = self.__fire_gitlab_api_request(method='POST', path=path, request_body=request_body)
         return self.__get_merge_request_from_json(mr)
 
     def __get_user_id_by_username(self, username: str) -> Optional[int]:
@@ -274,7 +267,7 @@ class GitLabClient(CodeHostingClient):
         request_body: Dict[str, str] = {'milestone_id': milestone}
         self.__fire_gitlab_api_project_request(method='PUT', path_suffix=f'/merge_requests/{number}', request_body=request_body)
 
-    def set_draft_status_of_pull_request(self, number: int, target_draft_status: bool) -> bool:
+    def set_draft_status_of_pull_request(self, number: int, *, target_draft_status: bool) -> bool:
         mr = self.__fire_gitlab_api_project_request(method='GET', path_suffix=f'/merge_requests/{number}')
         is_draft = bool(mr["draft"])  # bool(...) to satisfy mypy
         if is_draft and target_draft_status is True:
@@ -333,3 +326,41 @@ class GitLabClient(CodeHostingClient):
     def get_ref_name_for_pull_request(self, mr_number: int) -> str:
         # See `git ls-remote` for any GitLab remote.
         return f"merge-requests/{mr_number}/head"
+
+
+GITLAB_CLIENT_SPEC = CodeHostingSpec(
+    base_branch_name='target',
+    client_class=GitLabClient,
+    default_domain=GitLabClient.DEFAULT_GITLAB_DOMAIN,
+    display_name='GitLab',
+    git_machete_command='gitlab',
+    head_branch_name='source',
+    organization_name='namespace',
+    # https://docs.gitlab.com/ee/user/project/description_templates.html#create-a-merge-request-template
+    # https://docs.gitlab.com/ee/user/project/description_templates.html#set-a-default-template-for-merge-requests-and-issues
+    # Actual MR template resolution for GitLab has a complex hierarchy of templates, including
+    # project-level, group-level and instance-level templates - and also the ability to "choose" a template.
+    # To keep things simple, we'll only support the "Default.md" template for now.
+    pr_description_path=['.gitlab', 'merge_request_templates', 'Default.md'],
+    pr_full_name='merge request',
+    pr_intro_br_before_branches=True,
+    pr_intro_explicit_title=True,
+    pr_ordinal_char='!',
+    pr_short_name='MR',
+    pr_short_name_article='an',
+    repository_name='project',
+    token_providers_message=(
+        f'\n\t1. `{GITLAB_TOKEN_ENV_VAR}` environment variable\n'
+        '\t2. Content of the `~/.gitlab-token` file\n'
+        '\t3. Current auth token from the `glab` GitLab CLI\n'
+    ),
+    git_config_keys=CodeHostingGitConfigKeys(
+        domain='machete.gitlab.domain',
+        organization='machete.gitlab.namespace',
+        repository='machete.gitlab.project',
+        remote='machete.gitlab.remote',
+        annotate_with_urls='machete.gitlab.annotateWithUrls',
+        force_description_from_commit_message='machete.gitlab.forceDescriptionFromCommitMessage',
+        pr_description_intro_style='machete.gitlab.mrDescriptionIntroStyle',
+    )
+)
