@@ -11,10 +11,7 @@ from .base_test import BaseTest
 
 
 class TestCLI(BaseTest):
-    def test_aliases(self) -> None:
-        """
-        Verify that each command alias is unique
-        """
+    def test_aliases_unique(self) -> None:
         assert len(alias_by_command.values()) == len(set(alias_by_command.values()))
 
     def test_main(self, mocker: MockerFixture) -> None:
@@ -23,6 +20,11 @@ class TestCLI(BaseTest):
             main()
         assert ExitCode.ARGUMENT_ERROR == e.value.code
 
+        with pytest.raises(SystemExit) as e:
+            self.patch_symbol(mocker, "sys.argv", ["", "status", "--", "--patch"])
+            main()
+        assert ExitCode.ARGUMENT_ERROR == e.value.code
+
         os.chdir(mkdtemp())
 
         with pytest.raises(SystemExit) as e:
