@@ -1,25 +1,29 @@
 import os
-from tempfile import mkdtemp
+from typing import Any, Dict, List
 
 from pytest_mock import MockerFixture
 
 from git_machete.code_hosting import OrganizationAndRepository
 from git_machete.github import GitHubClient
-from tests.base_test import BaseTest, GitRepositorySandbox
-from tests.mockers import (assert_failure, assert_success, launch_command,
-                           rewrite_branch_layout_file)
+from tests.base_test import BaseTest
+from tests.mockers import (assert_failure, assert_success, execute,
+                           launch_command, rewrite_branch_layout_file)
 from tests.mockers_code_hosting import mock_from_url
+from tests.mockers_git_repository import (add_remote, check_out, commit,
+                                          create_repo, create_repo_with_remote,
+                                          delete_branch, new_branch, push,
+                                          remove_remote, set_git_config_key,
+                                          set_remote_url)
 from tests.mockers_github import (MockGitHubAPIState,
                                   mock_github_token_for_domain_fake,
                                   mock_github_token_for_domain_none,
-                                  mock_pr_json, mock_repositories,
-                                  mock_urlopen)
+                                  mock_pr_json, mock_urlopen)
 
 
 class TestGitHubCheckoutPRs(BaseTest):
     @staticmethod
-    def github_api_state_for_test_checkout_prs() -> MockGitHubAPIState:
-        return MockGitHubAPIState(
+    def prs_for_test_checkout_prs() -> List[Dict[str, Any]]:
+        return [
             mock_pr_json(head='chore/redundant_checks', base='restrict_access', number=18),
             mock_pr_json(head='restrict_access', base='allow-ownership-link', number=17),
             mock_pr_json(head='allow-ownership-link', base='bugfix/feature', number=12),
@@ -29,66 +33,77 @@ class TestGitHubCheckoutPRs(BaseTest):
             mock_pr_json(head='chore/comments', base='testing/add_user', number=24),
             mock_pr_json(head='ignore-trailing', base='hotfix/add-trigger', number=3),
             mock_pr_json(head='bugfix/remove-n-option', base='develop', number=5, state='closed', repo_id=2)
-        )
+        ]
+
+    @staticmethod
+    def repositories_for_test_github_checkout_prs(second_remote_path: str) -> Dict[int, Dict[str, Any]]:
+        return {
+            1: {'owner': {'login': 'tester'}, 'name': 'repo_sandbox', 'clone_url': 'https://github.com/tester/repo_sandbox.git'},
+            2: {'owner': {'login': 'tester'}, 'name': 'repo_sandbox', 'clone_url': second_remote_path},
+        }
 
     def test_github_checkout_prs(self, mocker: MockerFixture) -> None:
         self.patch_symbol(mocker, 'git_machete.code_hosting.OrganizationAndRepository.from_url', mock_from_url)
         self.patch_symbol(mocker, 'git_machete.github.GitHubToken.for_domain', mock_github_token_for_domain_none)
-        self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(self.github_api_state_for_test_checkout_prs()))
+        (local_path, remote_path) = create_repo_with_remote()
+        second_remote_path = create_repo("second-remote", bare=True, switch_dir_to_new_repo=False)
+        github_api_state = MockGitHubAPIState(
+            self.repositories_for_test_github_checkout_prs(second_remote_path),
+            *self.prs_for_test_checkout_prs())
+        self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(github_api_state))
+
+        new_branch("root")
+        commit("initial commit")
+        new_branch("develop")
+        commit("first commit")
+        push()
+        new_branch("enhance/feature")
+        commit("introduce feature")
+        push()
+        new_branch("bugfix/feature")
+        commit("bugs removed")
+        push()
+        new_branch("allow-ownership-link")
+        commit("fixes")
+        push()
+        new_branch('restrict_access')
+        commit('authorized users only')
+        push()
+        new_branch("chore/redundant_checks")
+        commit('remove some checks')
+        push()
+        check_out("root")
+        new_branch("master")
+        commit("Master commit")
+        push()
+        new_branch("hotfix/add-trigger")
+        commit("HOTFIX Add the trigger")
+        push()
+        new_branch("ignore-trailing")
+        commit("Ignore trailing data")
+        push()
+        delete_branch("root")
+        new_branch('chore/fields')
+        commit("remove outdated fields")
+        push()
+        check_out('develop')
+        new_branch('enhance/add_user')
+        commit('allow externals to add users')
+        push()
+        new_branch('bugfix/add_user')
+        commit('first round of fixes')
+        push()
+        new_branch('testing/add_user')
+        commit('add test set for add_user feature')
+        push()
+        new_branch('chore/comments')
+        commit('code maintenance')
+        push()
+        check_out('master')
 
-        (
-            self.repo_sandbox.new_branch("root")
-            .commit("initial commit")
-            .new_branch("develop")
-            .commit("first commit")
-            .push()
-            .new_branch("enhance/feature")
-            .commit("introduce feature")
-            .push()
-            .new_branch("bugfix/feature")
-            .commit("bugs removed")
-            .push()
-            .new_branch("allow-ownership-link")
-            .commit("fixes")
-            .push()
-            .new_branch('restrict_access')
-            .commit('authorized users only')
-            .push()
-            .new_branch("chore/redundant_checks")
-            .commit('remove some checks')
-            .push()
-            .check_out("root")
-            .new_branch("master")
-            .commit("Master commit")
-            .push()
-            .new_branch("hotfix/add-trigger")
-            .commit("HOTFIX Add the trigger")
-            .push()
-            .new_branch("ignore-trailing")
-            .commit("Ignore trailing data")
-            .push()
-            .delete_branch("root")
-            .new_branch('chore/fields')
-            .commit("remove outdated fields")
-            .push()
-            .check_out('develop')
-            .new_branch('enhance/add_user')
-            .commit('allow externals to add users')
-            .push()
-            .new_branch('bugfix/add_user')
-            .commit('first round of fixes')
-            .push()
-            .new_branch('testing/add_user')
-            .commit('add test set for add_user feature')
-            .push()
-            .new_branch('chore/comments')
-            .commit('code maintenance')
-            .push()
-            .check_out('master')
-        )
         for branch in ('chore/redundant_checks', 'restrict_access', 'allow-ownership-link', 'bugfix/feature', 'enhance/add_user',
                        'testing/add_user', 'chore/comments', 'bugfix/add_user'):
-            self.repo_sandbox.delete_branch(branch)
+            delete_branch(branch)
 
         body: str = \
             """
@@ -98,10 +113,6 @@ class TestGitHubCheckoutPRs(BaseTest):
                         chore/fields
             develop
                 enhance/feature
-                    bugfix/feature
-                        allow-ownership-link
-                            restrict_access
-                                chore/redundant_checks
             """
         rewrite_branch_layout_file(body)
 
@@ -114,7 +125,7 @@ class TestGitHubCheckoutPRs(BaseTest):
             |
             o-hotfix/add-trigger
               |
-              o-ignore-trailing  PR #3 (some_other_user) rebase=no push=no
+              o-ignore-trailing
                 |
                 o-chore/fields
 
@@ -140,7 +151,7 @@ class TestGitHubCheckoutPRs(BaseTest):
             |
             o-hotfix/add-trigger
               |
-              o-ignore-trailing  PR #3 (some_other_user) rebase=no push=no
+              o-ignore-trailing
                 |
                 o-chore/fields
 
@@ -173,7 +184,7 @@ class TestGitHubCheckoutPRs(BaseTest):
             |
             o-hotfix/add-trigger
               |
-              o-ignore-trailing  PR #3 (some_other_user) rebase=no push=no
+              o-ignore-trailing
                 |
                 o-chore/fields
 
@@ -235,7 +246,7 @@ class TestGitHubCheckoutPRs(BaseTest):
         # check against wrong PR number
         org_repo = OrganizationAndRepository.from_url(
             domain=GitHubClient.DEFAULT_GITHUB_DOMAIN,
-            url=self.repo_sandbox.remote_path)
+            url=remote_path)
 
         assert org_repo is not None
         expected_error_message = f"PR #100 is not found in repository {org_repo.organization}/{org_repo.repository}"
@@ -247,29 +258,22 @@ class TestGitHubCheckoutPRs(BaseTest):
         assert_success(['github', 'checkout-prs', '--by', 'some_other_user'], expected_msg)
 
         # Check against closed pull request with head branch deleted from remote
-        other_local_path = mkdtemp()
-        self.repo_sandbox.new_repo(GitRepositorySandbox.second_remote_path, bare=True)
-        (
-            self.repo_sandbox
-            .new_repo(other_local_path, bare=False)
-            .add_remote("origin", GitRepositorySandbox.second_remote_path)
-            .new_branch('main')
-            .commit('initial commit')
-            .push()
-        )
+        create_repo("other-local", bare=False, switch_dir_to_new_repo=True)
+
+        add_remote("origin", second_remote_path)
+        new_branch('main')
+        commit('initial commit')
+        push()
 
         expected_error_message = "Could not check out PR #5 because branch bugfix/remove-n-option " \
                                  "is already deleted from tester."
         assert_failure(['github', 'checkout-prs', '5'], expected_error_message)
 
         # Check against PR coming from fork
-        (
-            self.repo_sandbox
-            .new_branch('bugfix/remove-n-option')
-            .commit('first commit')
-            .push()
-            .chdir(self.repo_sandbox.local_path)
-        )
+        new_branch('bugfix/remove-n-option')
+        commit('first commit')
+        push()
+        os.chdir(local_path)
 
         expected_msg = ("Checking for open GitHub PRs... OK\n"
                         "Warn: PR #5 is already closed.\n"
@@ -280,136 +284,30 @@ class TestGitHubCheckoutPRs(BaseTest):
         expected_msg = 'Checking for open GitHub PRs... OK\n'
         assert_success(['github', 'checkout-prs', '3', '12'], expected_msg)
 
-    @staticmethod
-    def github_api_state_for_test_github_checkout_prs_fresh_repo() -> MockGitHubAPIState:
-        return MockGitHubAPIState(
-            mock_pr_json(head='comments/add_docstrings', base='improve/refactor', number=2),
-            mock_pr_json(head='restrict_access', base='allow-ownership-link', number=17),
-            mock_pr_json(head='improve/refactor', base='chore/sync_to_docs', number=1),
-            mock_pr_json(head='sphinx_export', base='comments/add_docstrings', number=23, state='closed', repo_id=2)
-        )
-
-    def test_github_checkout_prs_freshly_cloned(self, mocker: MockerFixture) -> None:
-        self.patch_symbol(mocker, 'git_machete.code_hosting.OrganizationAndRepository.from_url', mock_from_url)
-        self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(self.github_api_state_for_test_github_checkout_prs_fresh_repo()))
-
-        (
-            self.repo_sandbox.new_branch("root")
-            .commit("initial commit")
-            .new_branch("develop")
-            .commit("first commit")
-            .push()
-            .new_branch("chore/sync_to_docs")
-            .commit("synchronize docs")
-            .push()
-            .new_branch("improve/refactor")
-            .commit("refactor code")
-            .push()
-            .new_branch("comments/add_docstrings")
-            .commit("docstring added")
-            .push()
-            .new_branch("sphinx_export")
-            .commit("export docs to html")
-            .push()
-            .check_out("root")
-            .new_branch("master")
-            .commit("Master commit")
-            .push()
-            .delete_branch("root")
-            .push()
-        )
-        for branch in ('develop', 'chore/sync_to_docs', 'improve/refactor', 'comments/add_docstrings'):
-            self.repo_sandbox.delete_branch(branch)
-        local_path = mkdtemp()
-        self.repo_sandbox\
-            .chdir(local_path)\
-            .execute(f'git clone {self.repo_sandbox.remote_path}')\
-            .chdir(os.path.join(local_path, os.listdir()[0]))
-
-        for branch in ('develop', 'chore/sync_to_docs', 'improve/refactor', 'comments/add_docstrings'):
-            self.repo_sandbox.delete_remote_branch(f"origin/{branch}")
-
-        self.repo_sandbox.new_repo(GitRepositorySandbox.second_remote_path, bare=True)
-        (
-            self.repo_sandbox.new_repo(local_path, bare=False)
-            .add_remote("origin", GitRepositorySandbox.second_remote_path)
-            .new_branch('feature')
-            .commit('initial commit')
-            .push()
-        )
-        os.chdir(self.repo_sandbox.local_path)
-        rewrite_branch_layout_file("master")
-        expected_msg = ("Checking for open GitHub PRs... OK\n"
-                        "PR #1 checked out at local branch improve/refactor\n"
-                        "PR #2 checked out at local branch comments/add_docstrings\n")
-        assert_success(
-            ['github', 'checkout-prs', '2'],
-            expected_msg
-        )
-
-        assert_success(
-            ["status"],
-            """
-            master
-
-            chore/sync_to_docs
-            |
-            o-improve/refactor  PR #1 (some_other_user) rebase=no push=no
-              |
-              o-comments/add_docstrings *  PR #2 (some_other_user) rebase=no push=no
-            """
-        )
-
-        # Check against closed pull request
-        self.repo_sandbox.delete_branch('sphinx_export')
-        expected_msg = ("Checking for open GitHub PRs... OK\n"
-                        "Warn: PR #23 is already closed.\n"
-                        "PR #23 checked out at local branch sphinx_export\n")
-
-        assert_success(
-            ['github', 'checkout-prs', '23'],
-            expected_msg
-        )
-        assert_success(
-            ["status"],
-            """
-            master
-
-            chore/sync_to_docs
-            |
-            o-improve/refactor  PR #1 (some_other_user) rebase=no push=no
-              |
-              o-comments/add_docstrings  PR #2 (some_other_user) rebase=no push=no
-                |
-                o-sphinx_export *
-            """
-        )
-
     @staticmethod
     def github_api_state_for_test_github_checkout_prs_from_fork_with_deleted_repo() -> MockGitHubAPIState:
-        return MockGitHubAPIState(
+        return MockGitHubAPIState.with_prs(
             mock_pr_json(head='feature/allow_checkout', base='develop', number=2, repo_id=0, state='closed'),
             mock_pr_json(head='bugfix/allow_checkout', base='develop', number=3)
         )
 
     def test_github_checkout_prs_from_fork_with_deleted_repo(self, mocker: MockerFixture) -> None:
         self.patch_symbol(mocker, 'git_machete.code_hosting.OrganizationAndRepository.from_url', mock_from_url)
+        self.patch_symbol(mocker, "git_machete.github.GitHubToken.for_domain", mock_github_token_for_domain_fake)
         self.patch_symbol(mocker, 'urllib.request.urlopen',
                           mock_urlopen(self.github_api_state_for_test_github_checkout_prs_from_fork_with_deleted_repo()))
 
-        (
-            self.repo_sandbox.new_branch("root")
-            .commit('initial master commit')
-            .push()
-            .new_branch('develop')
-            .commit('initial develop commit')
-            .push()
-        )
+        (local_path, remote_path) = create_repo_with_remote()
+        new_branch("root")
+        commit('initial master commit')
+        push()
+        new_branch('develop')
+        commit('initial develop commit')
+        push()
 
-        self.repo_sandbox \
-            .chdir(self.repo_sandbox.remote_path)\
-            .execute("git branch pull/2/head develop")\
-            .chdir(self.repo_sandbox.local_path)
+        os.chdir(remote_path)
+        execute("git branch pull/2/head develop")
+        os.chdir(local_path)
 
         body: str = \
             """
@@ -433,7 +331,7 @@ class TestGitHubCheckoutPRs(BaseTest):
 
     @staticmethod
     def github_api_state_for_test_github_checkout_prs_of_current_user_and_other_users() -> MockGitHubAPIState:
-        return MockGitHubAPIState(
+        return MockGitHubAPIState.with_prs(
             mock_pr_json(head='chore/redundant_checks', base='restrict_access', number=18),
             mock_pr_json(head='restrict_access', base='allow-ownership-link', number=17, user='github_user'),
             mock_pr_json(head='allow-ownership-link', base='bugfix/feature', number=12),
@@ -451,59 +349,59 @@ class TestGitHubCheckoutPRs(BaseTest):
         self.patch_symbol(mocker, 'urllib.request.urlopen',
                           mock_urlopen(self.github_api_state_for_test_github_checkout_prs_of_current_user_and_other_users()))
 
-        (
-            self.repo_sandbox.new_branch("root")
-            .commit("initial commit")
-            .new_branch("develop")
-            .commit("first commit")
-            .push()
-            .new_branch("enhance/feature")
-            .commit("introduce feature")
-            .push()
-            .new_branch("bugfix/feature")
-            .commit("bugs removed")
-            .push()
-            .new_branch("allow-ownership-link")
-            .commit("fixes")
-            .push()
-            .new_branch('restrict_access')
-            .commit('authorized users only')
-            .push()
-            .new_branch("chore/redundant_checks")
-            .commit('remove some checks')
-            .push()
-            .check_out("root")
-            .new_branch("master")
-            .commit("Master commit")
-            .push()
-            .new_branch("hotfix/add-trigger")
-            .commit("HOTFIX Add the trigger")
-            .push()
-            .new_branch("ignore-trailing")
-            .commit("Ignore trailing data")
-            .push()
-            .delete_branch("root")
-            .new_branch('chore/fields')
-            .commit("remove outdated fields")
-            .push()
-            .check_out('develop')
-            .new_branch('enhance/add_user')
-            .commit('allow externals to add users')
-            .push()
-            .new_branch('bugfix/add_user')
-            .commit('first round of fixes')
-            .push()
-            .new_branch('testing/add_user')
-            .commit('add test set for add_user feature')
-            .push()
-            .new_branch('chore/comments')
-            .commit('code maintenance')
-            .push()
-            .check_out('master')
-        )
+        create_repo_with_remote()
+        new_branch("root")
+        commit("initial commit")
+        new_branch("develop")
+        commit("first commit")
+        push()
+        new_branch("enhance/feature")
+        commit("introduce feature")
+        push()
+        new_branch("bugfix/feature")
+        commit("bugs removed")
+        push()
+        new_branch("allow-ownership-link")
+        commit("fixes")
+        push()
+        new_branch('restrict_access')
+        commit('authorized users only')
+        push()
+        new_branch("chore/redundant_checks")
+        commit('remove some checks')
+        push()
+        check_out("root")
+        new_branch("master")
+        commit("Master commit")
+        push()
+        new_branch("hotfix/add-trigger")
+        commit("HOTFIX Add the trigger")
+        push()
+        new_branch("ignore-trailing")
+        commit("Ignore trailing data")
+        push()
+        delete_branch("root")
+        new_branch('chore/fields')
+        commit("remove outdated fields")
+        push()
+        check_out('develop')
+        new_branch('enhance/add_user')
+        commit('allow externals to add users')
+        push()
+        new_branch('bugfix/add_user')
+        commit('first round of fixes')
+        push()
+        new_branch('testing/add_user')
+        commit('add test set for add_user feature')
+        push()
+        new_branch('chore/comments')
+        commit('code maintenance')
+        push()
+        check_out('master')
+
         for branch in ('chore/redundant_checks', 'restrict_access', 'allow-ownership-link', 'bugfix/feature', 'enhance/add_user',
                        'testing/add_user', 'chore/comments', 'bugfix/add_user'):
-            self.repo_sandbox.delete_branch(branch)
+            delete_branch(branch)
 
         body: str = \
             """
@@ -513,13 +411,6 @@ class TestGitHubCheckoutPRs(BaseTest):
                         chore/fields
             develop
                 enhance/feature
-                    bugfix/feature
-                        allow-ownership-link
-                            restrict_access
-                                chore/redundant_checks
-            bugfix/add_user
-                testing/add_user
-                    chore/comments
             """
         rewrite_branch_layout_file(body)
 
@@ -596,8 +487,9 @@ class TestGitHubCheckoutPRs(BaseTest):
         )
 
     def test_github_checkout_prs_misc_failures_and_warns(self, mocker: MockerFixture) -> None:
+        create_repo_with_remote()
         self.patch_symbol(mocker, 'git_machete.code_hosting.OrganizationAndRepository.from_url', mock_from_url)
-        self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(MockGitHubAPIState()))
+        self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(MockGitHubAPIState.with_prs()))
 
         self.patch_symbol(mocker, 'git_machete.github.GitHubToken.for_domain', mock_github_token_for_domain_none)
         assert_success(
@@ -632,13 +524,13 @@ class TestGitHubCheckoutPRs(BaseTest):
             ["github", "checkout-prs", "--mine"],
             """
             Checking for open GitHub PRs... OK
-            Warn: Current user github_user has no open pull request in repository example-org/example-repo
+            Warn: User github_user has no open pull request in repository example-org/example-repo
             """
         )
 
     @staticmethod
     def github_api_state_with_pr_cycle() -> MockGitHubAPIState:
-        return MockGitHubAPIState(
+        return MockGitHubAPIState.with_prs(
             mock_pr_json(head='bugfix/feature', base='chore/redundant_checks', number=6),
             mock_pr_json(head='chore/redundant_checks', base='restrict_access', number=18),
             mock_pr_json(head='restrict_access', base='allow-ownership-link', number=17),
@@ -647,23 +539,22 @@ class TestGitHubCheckoutPRs(BaseTest):
 
     def test_github_checkout_prs_forming_a_cycle(self, mocker: MockerFixture) -> None:
         self.patch_symbol(mocker, 'git_machete.code_hosting.OrganizationAndRepository.from_url', mock_from_url)
+        self.patch_symbol(mocker, "git_machete.github.GitHubToken.for_domain", mock_github_token_for_domain_fake)
         self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(self.github_api_state_with_pr_cycle()))
 
-        (
-            self.repo_sandbox
-            .new_branch("bugfix/feature")
-            .commit("bugs removed")
-            .push()
-            .new_branch("allow-ownership-link")
-            .commit("fixes")
-            .push()
-            .new_branch('restrict_access')
-            .commit('authorized users only')
-            .push()
-            .new_branch("chore/redundant_checks")
-            .commit('remove some checks')
-            .push()
-        )
+        create_repo_with_remote()
+        new_branch("bugfix/feature")
+        commit("bugs removed")
+        push()
+        new_branch("allow-ownership-link")
+        commit("fixes")
+        push()
+        new_branch('restrict_access')
+        commit('authorized users only')
+        push()
+        new_branch("chore/redundant_checks")
+        commit('remove some checks')
+        push()
 
         assert_failure(
             ['github', 'checkout-prs', '--all'],
@@ -673,29 +564,27 @@ class TestGitHubCheckoutPRs(BaseTest):
 
     @staticmethod
     def github_api_state_for_test_github_checkout_prs_single_pr() -> MockGitHubAPIState:
-        return MockGitHubAPIState(
+        return MockGitHubAPIState.with_prs(
             mock_pr_json(head='develop', base='master', number=18)
         )
 
     def test_github_checkout_prs_remote_already_added(self, mocker: MockerFixture) -> None:
         self.patch_symbol(mocker, 'git_machete.git_operations.GitContext.fetch_remote', lambda _self, _remote: None)
-        self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(
-            self.github_api_state_for_test_github_checkout_prs_single_pr()))
-        (
-            self.repo_sandbox
-            .remove_remote("origin")
-            .add_remote("origin-1", mock_repositories()[1]['clone_url'])
-        )
+        self.patch_symbol(mocker, "git_machete.github.GitHubToken.for_domain", mock_github_token_for_domain_fake)
+        github_api_state = self.github_api_state_for_test_github_checkout_prs_single_pr()
+        self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(github_api_state))
+
+        create_repo()
+        add_remote("origin-1", github_api_state.repositories[1]['clone_url'])
+
         assert_failure(
             ["github", "checkout-prs", "--all"],
             "Could not check out PR #18 because branch develop is already deleted from origin-1."
         )
 
-        (
-            self.repo_sandbox
-            .remove_remote("origin-1")
-            .add_remote("tester", 'https://github.com/tester/lolxd.git')
-        )
+        remove_remote("origin-1")
+        add_remote("tester", 'https://github.com/tester/lolxd.git')
+
         assert_failure(
             ["github", "checkout-prs", "--all"],
             "Could not check out PR #18 because branch develop is already deleted from tester."
@@ -703,18 +592,21 @@ class TestGitHubCheckoutPRs(BaseTest):
 
     def test_github_checkout_prs_org_and_repo_from_config(self, mocker: MockerFixture) -> None:
         self.patch_symbol(mocker, 'git_machete.git_operations.GitContext.fetch_remote', lambda _self, _remote: None)
+        self.patch_symbol(mocker, "git_machete.github.GitHubToken.for_domain", mock_github_token_for_domain_none)
         self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(
             self.github_api_state_for_test_github_checkout_prs_single_pr()))
 
-        (
-            self.repo_sandbox
-            .new_branch("master").commit()
-            .new_branch("develop").commit().push()
-        )
+        create_repo_with_remote()
+        new_branch("master")
+        commit()
 
-        self.repo_sandbox.set_remote_url("origin", "https://github.com/example-org/example-repo.git")
-        self.repo_sandbox.set_git_config_key('machete.github.organization', "example-org")
-        self.repo_sandbox.set_git_config_key('machete.github.repository', "example-repo")
+        new_branch("develop")
+        commit()
+        push()
+
+        set_remote_url("origin", "https://github.com/example-org/example-repo.git")
+        set_git_config_key('machete.github.organization', "example-org")
+        set_git_config_key('machete.github.repository', "example-repo")
         assert_success(
             ['github', 'checkout-prs', '--all'],
             'Checking for open GitHub PRs... OK\n'
@@ -723,19 +615,54 @@ class TestGitHubCheckoutPRs(BaseTest):
 
     def test_github_checkout_prs_remote_from_config(self, mocker: MockerFixture) -> None:
         self.patch_symbol(mocker, 'git_machete.git_operations.GitContext.fetch_remote', lambda _self, _remote: None)
+        self.patch_symbol(mocker, "git_machete.github.GitHubToken.for_domain", mock_github_token_for_domain_none)
         self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(
             self.github_api_state_for_test_github_checkout_prs_single_pr()))
 
-        (
-            self.repo_sandbox
-            .new_branch("master").commit()
-            .new_branch("develop").commit().push()
-        )
+        create_repo_with_remote()
+        new_branch("master")
+        commit()
 
-        self.repo_sandbox.set_remote_url("origin", "https://github.com/example-org/example-repo.git")
-        self.repo_sandbox.set_git_config_key('machete.github.remote', "origin")
+        new_branch("develop")
+        commit()
+        push()
+
+        set_remote_url("origin", "https://github.com/example-org/example-repo.git")
+        set_git_config_key('machete.github.remote', "origin")
         assert_success(
             ['github', 'checkout-prs', '--all'],
             'Checking for open GitHub PRs... OK\n'
             'PR #18 checked out at local branch develop\n'
         )
+
+    @staticmethod
+    def prs_for_test_checkout_prs_main_to_main_pr() -> List[Dict[str, Any]]:
+        return [
+            mock_pr_json(head='fix-10341', base='main', number=2),
+            mock_pr_json(head='main', base='main', number=1, repo_id=2)
+        ]
+
+    def test_github_checkout_prs_main_to_main_pr(self, mocker: MockerFixture) -> None:
+        self.patch_symbol(mocker, 'git_machete.git_operations.GitContext.fetch_remote', lambda _self, _remote: None)
+        self.patch_symbol(mocker, 'git_machete.code_hosting.OrganizationAndRepository.from_url', mock_from_url)
+        self.patch_symbol(mocker, 'git_machete.github.GitHubToken.for_domain', mock_github_token_for_domain_none)
+
+        create_repo_with_remote()
+        second_remote_path = create_repo("second-remote", bare=True, switch_dir_to_new_repo=False)
+        github_api_state = MockGitHubAPIState(
+            self.repositories_for_test_github_checkout_prs(second_remote_path),
+            *self.prs_for_test_checkout_prs_main_to_main_pr())
+        self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(github_api_state))
+
+        new_branch("main")
+        commit()
+        push()
+        new_branch("fix-10341")
+        commit()
+        push()
+
+        assert_success(
+            ['github', 'checkout-prs', '2'],
+            'Checking for open GitHub PRs... OK\n'
+            'PR #2 checked out at local branch fix-10341\n'
+        )
