@@ -13,6 +13,11 @@ from tests.mockers import (assert_failure, assert_success, launch_command,
                            mock_input_returning_y, overridden_environment,
                            rewrite_branch_layout_file)
 from tests.mockers_code_hosting import mock_from_url, mock_shutil_which
+from tests.mockers_git_repository import (add_remote, check_out, commit,
+                                          create_repo_with_remote,
+                                          delete_branch, new_branch, push,
+                                          set_git_config_key,
+                                          unset_git_config_key)
 from tests.mockers_gitlab import (MockGitLabAPIState,
                                   mock_gitlab_token_for_domain_fake,
                                   mock_gitlab_token_for_domain_none,
@@ -23,7 +28,7 @@ class TestGitLab(BaseTest):
 
     def test_gitlab_client_constructor(self) -> None:
         # This is solely to make mypy check if the class correctly implements abstract methods from CodeHostingClient.
-        GitLabClient(GitLabClient.spec(), "gitlab.com", "my-org", "my-repo")
+        GitLabClient(domain="gitlab.com", organization="my-org", repository="my-repo")
 
     def test_gitlab_remote_patterns(self) -> None:
         organization = 'virtuslab'
@@ -31,7 +36,7 @@ class TestGitLab(BaseTest):
         urls = [f'https://tester@gitlab.com/{organization}/{repository}',
                 f'https://gitlab.com/{organization}/{repository}',
                 f'git@gitlab.com:{organization}/{repository}',
-                f'ssh://git@gitlab.com/{organization}/{repository}']
+                f'ssh://foo-3@gitlab.com/{organization}/{repository}']
         urls = urls + [url + '.git' for url in urls]
 
         for url in urls:
@@ -40,14 +45,14 @@ class TestGitLab(BaseTest):
             assert org_and_repo.organization == organization
             assert org_and_repo.repository == repository
 
-    PR_COUNT_FOR_TEST_GITLAB_API_PAGINATION = 12
+    MR_COUNT_FOR_TEST_GITLAB_API_PAGINATION = 12
 
     @staticmethod
     def gitlab_api_state_for_test_gitlab_api_pagination() -> MockGitLabAPIState:
-        return MockGitLabAPIState(*[
+        return MockGitLabAPIState.with_mrs(*[
             mock_mr_json(
                 head=f'feature_{i:02d}', base='develop', number=i
-            ) for i in range(TestGitLab.PR_COUNT_FOR_TEST_GITLAB_API_PAGINATION)])
+            ) for i in range(TestGitLab.MR_COUNT_FOR_TEST_GITLAB_API_PAGINATION)])
 
     def test_gitlab_api_pagination(self, mocker: MockerFixture) -> None:
         self.patch_symbol(mocker, 'builtins.input', mock_input_returning_y)
@@ -56,35 +61,41 @@ class TestGitLab(BaseTest):
         self.patch_symbol(mocker, 'git_machete.code_hosting.OrganizationAndRepository.from_url', mock_from_url)
         self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(self.gitlab_api_state_for_test_gitlab_api_pagination()))
 
-        (
-            self.repo_sandbox.new_branch("develop")
-            .commit("first commit")
-            .push()
-        )
-        for i in range(self.PR_COUNT_FOR_TEST_GITLAB_API_PAGINATION):
-            self.repo_sandbox.check_out('develop').new_branch(f'feature_{i:02d}').commit().push()
-        self.repo_sandbox.check_out('develop')
-        body: str = 'develop *\n' + '\n'.join([f'feature_{i:02d}' for i in range(self.PR_COUNT_FOR_TEST_GITLAB_API_PAGINATION)]) + '\n'
+        create_repo_with_remote()
+        new_branch("develop")
+        commit("first commit")
+        push()
+
+        for i in range(self.MR_COUNT_FOR_TEST_GITLAB_API_PAGINATION):
+            check_out('develop')
+            new_branch(f'feature_{i:02d}')
+            commit()
+            push()
+
+        check_out('develop')
+        body: str = 'develop *\n' + '\n'.join([f'feature_{i:02d}' for i in range(self.MR_COUNT_FOR_TEST_GITLAB_API_PAGINATION)]) + '\n'
         rewrite_branch_layout_file(body)
 
-        self.repo_sandbox.check_out('develop')
-        for i in range(self.PR_COUNT_FOR_TEST_GITLAB_API_PAGINATION):
-            self.repo_sandbox.delete_branch(f"feature_{i:02d}")
+        check_out('develop')
+        for i in range(self.MR_COUNT_FOR_TEST_GITLAB_API_PAGINATION):
+            delete_branch(f"feature_{i:02d}")
         body = 'develop *\n'
         rewrite_branch_layout_file(body)
 
         launch_command('gitlab', 'checkout-mrs', '--all')
         launch_command('discover', '--checked-out-since=1 day ago')
         expected_status_output = 'develop *\n' + '\n'.join([f'|\no-feature_{i:02d}  rebase=no push=no'
-                                                            for i in range(self.PR_COUNT_FOR_TEST_GITLAB_API_PAGINATION)]) + '\n'
+                                                            for i in range(self.MR_COUNT_FOR_TEST_GITLAB_API_PAGINATION)]) + '\n'
         assert_success(['status'], expected_status_output)
 
     def test_gitlab_enterprise_domain_unauthorized_without_token(self, mocker: MockerFixture) -> None:
         self.patch_symbol(mocker, 'git_machete.gitlab.GitLabToken.for_domain', mock_gitlab_token_for_domain_none)
         self.patch_symbol(mocker, 'git_machete.code_hosting.OrganizationAndRepository.from_url', mock_from_url)
-        self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(MockGitLabAPIState()))
+        self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(MockGitLabAPIState.with_mrs()))
 
-        self.repo_sandbox.set_git_config_key('machete.gitlab.domain', '403.example.org')
+        create_repo_with_remote()
+        set_git_config_key('http.sslVerify', 'false')
+        set_git_config_key('machete.gitlab.domain', '403.example.org')
 
         expected_error_message = (
             "GitLab API returned 403 HTTP status with error message: Forbidden\n"
@@ -97,9 +108,10 @@ class TestGitLab(BaseTest):
     def test_gitlab_enterprise_domain_unauthorized_with_token(self, mocker: MockerFixture) -> None:
         self.patch_symbol(mocker, 'git_machete.gitlab.GitLabToken.for_domain', mock_gitlab_token_for_domain_fake)
         self.patch_symbol(mocker, 'git_machete.code_hosting.OrganizationAndRepository.from_url', mock_from_url)
-        self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(MockGitLabAPIState()))
+        self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(MockGitLabAPIState.with_mrs()))
 
-        self.repo_sandbox.set_git_config_key('machete.gitlab.domain', '403.example.org')
+        create_repo_with_remote()
+        set_git_config_key('machete.gitlab.domain', '403.example.org')
 
         expected_error_message = (
             "GitLab API returned 403 HTTP status with error message: Forbidden\n"
@@ -110,7 +122,7 @@ class TestGitLab(BaseTest):
 
     @staticmethod
     def gitlab_api_state_for_test_gitlab_enterprise_domain() -> MockGitLabAPIState:
-        return MockGitLabAPIState(
+        return MockGitLabAPIState.with_mrs(
             mock_mr_json(head='snickers', base='develop', number=7, user='gitlab_user')
         )
 
@@ -120,25 +132,27 @@ class TestGitLab(BaseTest):
         self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(self.gitlab_api_state_for_test_gitlab_enterprise_domain()))
 
         gitlab_enterprise_domain = 'git.example.org'
-        (
-            self.repo_sandbox.new_branch("develop")
-            .commit("first commit")
-            .push()
-            .new_branch("snickers")
-            .commit("first commit")
-            .push()
-            .check_out("develop")
-            .delete_branch("snickers")
-            .set_git_config_key('machete.gitlab.domain', gitlab_enterprise_domain)
-        )
+        create_repo_with_remote()
+        new_branch("develop")
+        commit("first commit")
+        push()
+        new_branch("snickers")
+        commit("second commit")
+        push()
+        check_out("develop")
+        delete_branch("snickers")
+        set_git_config_key('machete.gitlab.domain', gitlab_enterprise_domain)
+
         launch_command('gitlab', 'checkout-mrs', '--all')
 
     def test_gitlab_invalid_config(self) -> None:
+        create_repo_with_remote()
+
         @contextmanager
         def git_config_key(key: str, value: str) -> Iterator[None]:
-            self.repo_sandbox.set_git_config_key(key, value)
+            set_git_config_key(key, value)
             yield
-            self.repo_sandbox.unset_git_config_key(key)
+            unset_git_config_key(key)
 
         with git_config_key('machete.gitlab.namespace', "example-org"):
             assert_failure(
@@ -165,7 +179,7 @@ class TestGitLab(BaseTest):
                     'but no remote seems to correspond to example-org/example-repo (namespace/project) on GitLab.\n'
                     'Consider pointing to the remote via machete.gitlab.remote config key')
 
-        self.repo_sandbox.add_remote("new-origin", "https://github.com/example-org/example-repo.git")  # not a valid GitLab repo URL
+        add_remote("new-origin", "https://github.com/example-org/example-repo.git")  # not a valid GitLab project URL
         with git_config_key('machete.gitlab.remote', "new-origin"):
             assert_failure(
                 ['gitlab', 'checkout-mrs', '--all'],
@@ -178,16 +192,15 @@ class TestGitLab(BaseTest):
         self.patch_symbol(mocker, 'shutil.which', mock_shutil_which(None))
         self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(self.gitlab_api_state_for_test_gitlab_enterprise_domain()))
 
-        (
-            self.repo_sandbox.new_branch("develop")
-                .commit("first commit")
-                .push()
-                .new_branch("snickers")
-                .commit("first commit")
-                .push()
-                .check_out("develop")
-                .delete_branch("snickers")
-        )
+        create_repo_with_remote()
+        new_branch("develop")
+        commit("first commit")
+        push()
+        new_branch("snickers")
+        commit("second commit")
+        push()
+        check_out("develop")
+        delete_branch("snickers")
 
         expected_output = ["__get_token_from_env(cls=<class 'git_machete.gitlab.GitLabToken'>): "
                            "1. Trying to find token in `GITLAB_TOKEN` environment variable...",
@@ -284,18 +297,28 @@ class TestGitLab(BaseTest):
         assert_failure(["gitlab", "anno-mrs", "--draft"],
                        "--draft option is only valid with create-mr subcommand.")
         assert_failure(["gitlab", "anno-mrs", "--mine"],
-                       "--mine option is only valid with checkout-mrs subcommand.")
+                       "--mine option is only valid with checkout-mrs and update-mr-descriptions subcommands.")
         assert_failure(["gitlab", "retarget-mr", "123"],
-                       "request_id option is only valid with checkout-mrs subcommand.")
+                       "MR number is only valid with checkout-mrs subcommand.")
         assert_failure(["gitlab", "checkout-mrs", "-b", "foo"],
                        "--branch option is only valid with retarget-mr subcommand.")
         assert_failure(["gitlab", "create-mr", "--ignore-if-missing"],
                        "--ignore-if-missing option is only valid with retarget-mr subcommand.")
         assert_failure(["gitlab", "checkout-mrs", "--all", "--mine"],
-                       "checkout-mrs subcommand must take exactly one of the following options: --all, --by=..., --mine, mr-number(s)")
+                       "checkout-mrs subcommand must take exactly one "
+                       "of the following options: --all, --by=..., --mine, mr-number(s)")
         assert_failure(["gitlab", "checkout-mrs", "--title=foo"],
                        "--title option is only valid with create-mr subcommand.")
         assert_failure(["gitlab", "restack-mr", "--with-urls"],
                        "--with-urls option is only valid with anno-mrs subcommand.")
         assert_failure(["gitlab", "restack-mr", "--yes"],
                        "--yes option is only valid with create-mr subcommand.")
+        assert_failure(["gitlab", "create-mr", "--by=other-user"],
+                       "--by option is only valid with checkout-mrs and update-mr-descriptions subcommands.")
+        assert_failure(["gitlab", "checkout-mrs", "--related"],
+                       "--related option is only valid with update-mr-descriptions subcommand.")
+        assert_failure(["gitlab", "update-mr-descriptions", "--all", "--related"],
+                       "update-mr-descriptions subcommand must take exactly one "
+                       "of the following options: --all, --by=..., --mine, --related")
+        assert_failure(["gitlab", "update-mr-descriptions", "--update-related-descriptions"],
+                       "--update-related-descriptions option is only valid with create-mr, restack-mr and retarget-mr subcommands.")
