@@ -1,24 +1,28 @@
 import os
-from tempfile import mkdtemp
+from typing import Any, Dict, List
 
 from pytest_mock import MockerFixture
 
 from git_machete.code_hosting import OrganizationAndRepository
 from git_machete.gitlab import GitLabClient
-from tests.base_test import BaseTest, GitRepositorySandbox
-from tests.mockers import (assert_failure, assert_success, launch_command,
-                           rewrite_branch_layout_file)
+from tests.base_test import BaseTest
+from tests.mockers import (assert_failure, assert_success, execute,
+                           launch_command, rewrite_branch_layout_file)
 from tests.mockers_code_hosting import mock_from_url
+from tests.mockers_git_repository import (add_remote, check_out, commit,
+                                          create_repo, create_repo_with_remote,
+                                          delete_branch, new_branch, push,
+                                          set_git_config_key, set_remote_url)
 from tests.mockers_gitlab import (MockGitLabAPIState,
                                   mock_gitlab_token_for_domain_fake,
                                   mock_gitlab_token_for_domain_none,
-                                  mock_mr_json, mock_projects, mock_urlopen)
+                                  mock_mr_json, mock_urlopen)
 
 
 class TestGitLabCheckoutMRs(BaseTest):
     @staticmethod
-    def gitlab_api_state_for_test_checkout_mrs() -> MockGitLabAPIState:
-        return MockGitLabAPIState(
+    def mrs_for_test_checkout_mrs() -> List[Dict[str, Any]]:
+        return [
             mock_mr_json(head='chore/redundant_checks', base='restrict_access', number=18),
             mock_mr_json(head='restrict_access', base='allow-ownership-link', number=17),
             mock_mr_json(head='allow-ownership-link', base='bugfix/feature', number=12),
@@ -28,66 +32,80 @@ class TestGitLabCheckoutMRs(BaseTest):
             mock_mr_json(head='chore/comments', base='testing/add_user', number=24),
             mock_mr_json(head='ignore-trailing', base='hotfix/add-trigger', number=3),
             mock_mr_json(head='bugfix/remove-n-option', base='develop', number=5, state='closed', repo_id=2)
-        )
+        ]
+
+    @staticmethod
+    def projects_for_test_gitlab_checkout_prs(second_remote_path: str) -> Dict[int, Dict[str, Any]]:
+        return {
+            1: {'namespace': {'full_path': 'tester/tester'}, 'name': 'repo_sandbox',
+                'http_url_to_repo': 'https://gitlab.com/tester/tester/repo_sandbox.git'},
+            2: {'namespace': {'full_path': 'tester'}, 'name': 'repo_sandbox',
+                'http_url_to_repo': second_remote_path},
+        }
 
     def test_gitlab_checkout_mrs(self, mocker: MockerFixture) -> None:
         self.patch_symbol(mocker, 'git_machete.code_hosting.OrganizationAndRepository.from_url', mock_from_url)
         self.patch_symbol(mocker, 'git_machete.gitlab.GitLabToken.for_domain', mock_gitlab_token_for_domain_none)
-        self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(self.gitlab_api_state_for_test_checkout_mrs()))
 
-        (
-            self.repo_sandbox.new_branch("root")
-            .commit("initial commit")
-            .new_branch("develop")
-            .commit("first commit")
-            .push()
-            .new_branch("enhance/feature")
-            .commit("introduce feature")
-            .push()
-            .new_branch("bugfix/feature")
-            .commit("bugs removed")
-            .push()
-            .new_branch("allow-ownership-link")
-            .commit("fixes")
-            .push()
-            .new_branch('restrict_access')
-            .commit('authorized users only')
-            .push()
-            .new_branch("chore/redundant_checks")
-            .commit('remove some checks')
-            .push()
-            .check_out("root")
-            .new_branch("master")
-            .commit("Master commit")
-            .push()
-            .new_branch("hotfix/add-trigger")
-            .commit("HOTFIX Add the trigger")
-            .push()
-            .new_branch("ignore-trailing")
-            .commit("Ignore trailing data")
-            .push()
-            .delete_branch("root")
-            .new_branch('chore/fields')
-            .commit("remove outdated fields")
-            .push()
-            .check_out('develop')
-            .new_branch('enhance/add_user')
-            .commit('allow externals to add users')
-            .push()
-            .new_branch('bugfix/add_user')
-            .commit('first round of fixes')
-            .push()
-            .new_branch('testing/add_user')
-            .commit('add test set for add_user feature')
-            .push()
-            .new_branch('chore/comments')
-            .commit('code maintenance')
-            .push()
-            .check_out('master')
-        )
+        (local_path, remote_path) = create_repo_with_remote()
+        second_remote_path = create_repo("second-remote", bare=True, switch_dir_to_new_repo=False)
+        gitlab_api_state = MockGitLabAPIState(
+            self.projects_for_test_gitlab_checkout_prs(second_remote_path),
+            *self.mrs_for_test_checkout_mrs())
+        self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(gitlab_api_state))
+
+        new_branch("root")
+        commit("initial commit")
+        new_branch("develop")
+        commit("first commit")
+        push()
+        new_branch("enhance/feature")
+        commit("introduce feature")
+        push()
+        new_branch("bugfix/feature")
+        commit("bugs removed")
+        push()
+        new_branch("allow-ownership-link")
+        commit("fixes")
+        push()
+        new_branch('restrict_access')
+        commit('authorized users only')
+        push()
+        new_branch("chore/redundant_checks")
+        commit('remove some checks')
+        push()
+        check_out("root")
+        new_branch("master")
+        commit("Master commit")
+        push()
+        new_branch("hotfix/add-trigger")
+        commit("HOTFIX Add the trigger")
+        push()
+        new_branch("ignore-trailing")
+        commit("Ignore trailing data")
+        push()
+        delete_branch("root")
+        new_branch('chore/fields')
+        commit("remove outdated fields")
+        push()
+        check_out('develop')
+        new_branch('enhance/add_user')
+        commit('allow externals to add users')
+        push()
+        new_branch('bugfix/add_user')
+        commit('first round of fixes')
+        push()
+        new_branch('testing/add_user')
+        commit('add test set for add_user feature')
+        push()
+        new_branch('chore/comments')
+        commit('code maintenance')
+        push()
+        check_out('master')
+
         for branch in ('chore/redundant_checks', 'restrict_access', 'allow-ownership-link', 'bugfix/feature', 'enhance/add_user',
                        'testing/add_user', 'chore/comments', 'bugfix/add_user'):
-            self.repo_sandbox.delete_branch(branch)
+            delete_branch(branch)
 
         body: str = \
             """
@@ -97,14 +115,10 @@ class TestGitLabCheckoutMRs(BaseTest):
                         chore/fields
             develop
                 enhance/feature
-                    bugfix/feature
-                        allow-ownership-link
-                            restrict_access
-                                chore/redundant_checks
             """
         rewrite_branch_layout_file(body)
 
-        # not broken chain of pull requests (root found in dependency tree)
+        # not broken chain of merge requests (root found in dependency tree)
         launch_command('gitlab', 'checkout-mrs', '18')
         assert_success(
             ["status"],
@@ -113,7 +127,7 @@ class TestGitLabCheckoutMRs(BaseTest):
             |
             o-hotfix/add-trigger
               |
-              o-ignore-trailing  MR !3 (some_other_user) rebase=no push=no
+              o-ignore-trailing
                 |
                 o-chore/fields
 
@@ -130,7 +144,7 @@ class TestGitLabCheckoutMRs(BaseTest):
                     o-chore/redundant_checks *  MR !18 (some_other_user) rebase=no push=no
             """
         )
-        # broken chain of pull requests (add new root)
+        # broken chain of merge requests (add new root)
         launch_command('gitlab', 'checkout-mrs', '24')
         assert_success(
             ["status"],
@@ -139,7 +153,7 @@ class TestGitLabCheckoutMRs(BaseTest):
             |
             o-hotfix/add-trigger
               |
-              o-ignore-trailing  MR !3 (some_other_user) rebase=no push=no
+              o-ignore-trailing
                 |
                 o-chore/fields
 
@@ -163,7 +177,7 @@ class TestGitLabCheckoutMRs(BaseTest):
             """
         )
 
-        # broken chain of pull requests (branches already added)
+        # broken chain of merge requests (branches already added)
         launch_command('gitlab', 'checkout-mrs', '24')
         assert_success(
             ["status"],
@@ -172,7 +186,7 @@ class TestGitLabCheckoutMRs(BaseTest):
             |
             o-hotfix/add-trigger
               |
-              o-ignore-trailing  MR !3 (some_other_user) rebase=no push=no
+              o-ignore-trailing
                 |
                 o-chore/fields
 
@@ -234,7 +248,7 @@ class TestGitLabCheckoutMRs(BaseTest):
         # check against wrong MR number
         org_repo = OrganizationAndRepository.from_url(
             domain=GitLabClient.DEFAULT_GITLAB_DOMAIN,
-            url=self.repo_sandbox.remote_path)
+            url=remote_path)
 
         assert org_repo is not None
         expected_error_message = f"MR !100 is not found in project {org_repo.organization}/{org_repo.repository}"
@@ -245,30 +259,22 @@ class TestGitLabCheckoutMRs(BaseTest):
         expected_msg = "Checking for open GitLab MRs... OK\n"
         assert_success(['gitlab', 'checkout-mrs', '--by', 'some_other_user'], expected_msg)
 
-        # Check against closed pull request with source branch deleted from remote
-        other_local_path = mkdtemp()
-        self.repo_sandbox.new_repo(GitRepositorySandbox.second_remote_path, bare=True)
-        (
-            self.repo_sandbox
-            .new_repo(other_local_path, bare=False)
-            .add_remote("origin", GitRepositorySandbox.second_remote_path)
-            .new_branch('main')
-            .commit('initial commit')
-            .push()
-        )
+        # Check against closed merge request with source branch deleted from remote
+        create_repo("other-local", bare=False, switch_dir_to_new_repo=True)
+        add_remote("origin", second_remote_path)
+        new_branch('main')
+        commit('initial commit')
+        push()
 
         expected_error_message = "Could not check out MR !5 because branch bugfix/remove-n-option " \
                                  "is already deleted from tester."
         assert_failure(['gitlab', 'checkout-mrs', '5'], expected_error_message)
 
         # Check against MR coming from fork
-        (
-            self.repo_sandbox
-            .new_branch('bugfix/remove-n-option')
-            .commit('first commit')
-            .push()
-            .chdir(self.repo_sandbox.local_path)
-        )
+        new_branch('bugfix/remove-n-option')
+        commit('first commit')
+        push()
+        os.chdir(local_path)
 
         expected_msg = ("Checking for open GitLab MRs... OK\n"
                         "Warn: MR !5 is already closed.\n"
@@ -279,136 +285,30 @@ class TestGitLabCheckoutMRs(BaseTest):
         expected_msg = 'Checking for open GitLab MRs... OK\n'
         assert_success(['gitlab', 'checkout-mrs', '3', '12'], expected_msg)
 
-    @staticmethod
-    def gitlab_api_state_for_test_gitlab_checkout_mrs_fresh_repo() -> MockGitLabAPIState:
-        return MockGitLabAPIState(
-            mock_mr_json(head='comments/add_docstrings', base='improve/refactor', number=2),
-            mock_mr_json(head='restrict_access', base='allow-ownership-link', number=17),
-            mock_mr_json(head='improve/refactor', base='chore/sync_to_docs', number=1),
-            mock_mr_json(head='sphinx_export', base='comments/add_docstrings', number=23, state='closed', repo_id=2)
-        )
-
-    def test_gitlab_checkout_mrs_freshly_cloned(self, mocker: MockerFixture) -> None:
-        self.patch_symbol(mocker, 'git_machete.code_hosting.OrganizationAndRepository.from_url', mock_from_url)
-        self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(self.gitlab_api_state_for_test_gitlab_checkout_mrs_fresh_repo()))
-
-        (
-            self.repo_sandbox.new_branch("root")
-            .commit("initial commit")
-            .new_branch("develop")
-            .commit("first commit")
-            .push()
-            .new_branch("chore/sync_to_docs")
-            .commit("synchronize docs")
-            .push()
-            .new_branch("improve/refactor")
-            .commit("refactor code")
-            .push()
-            .new_branch("comments/add_docstrings")
-            .commit("docstring added")
-            .push()
-            .new_branch("sphinx_export")
-            .commit("export docs to html")
-            .push()
-            .check_out("root")
-            .new_branch("master")
-            .commit("Master commit")
-            .push()
-            .delete_branch("root")
-            .push()
-        )
-        for branch in ('develop', 'chore/sync_to_docs', 'improve/refactor', 'comments/add_docstrings'):
-            self.repo_sandbox.delete_branch(branch)
-        local_path = mkdtemp()
-        self.repo_sandbox\
-            .chdir(local_path)\
-            .execute(f'git clone {self.repo_sandbox.remote_path}')\
-            .chdir(os.path.join(local_path, os.listdir()[0]))
-
-        for branch in ('develop', 'chore/sync_to_docs', 'improve/refactor', 'comments/add_docstrings'):
-            self.repo_sandbox.delete_remote_branch(f"origin/{branch}")
-
-        self.repo_sandbox.new_repo(GitRepositorySandbox.second_remote_path, bare=True)
-        (
-            self.repo_sandbox.new_repo(local_path, bare=False)
-            .add_remote("origin", GitRepositorySandbox.second_remote_path)
-            .new_branch('feature')
-            .commit('initial commit')
-            .push()
-        )
-        os.chdir(self.repo_sandbox.local_path)
-        rewrite_branch_layout_file("master")
-        expected_msg = ("Checking for open GitLab MRs... OK\n"
-                        "MR !1 checked out at local branch improve/refactor\n"
-                        "MR !2 checked out at local branch comments/add_docstrings\n")
-        assert_success(
-            ['gitlab', 'checkout-mrs', '2'],
-            expected_msg
-        )
-
-        assert_success(
-            ["status"],
-            """
-            master
-
-            chore/sync_to_docs
-            |
-            o-improve/refactor  MR !1 (some_other_user) rebase=no push=no
-              |
-              o-comments/add_docstrings *  MR !2 (some_other_user) rebase=no push=no
-            """
-        )
-
-        # Check against closed pull request
-        self.repo_sandbox.delete_branch('sphinx_export')
-        expected_msg = ("Checking for open GitLab MRs... OK\n"
-                        "Warn: MR !23 is already closed.\n"
-                        "MR !23 checked out at local branch sphinx_export\n")
-
-        assert_success(
-            ['gitlab', 'checkout-mrs', '23'],
-            expected_msg
-        )
-        assert_success(
-            ["status"],
-            """
-            master
-
-            chore/sync_to_docs
-            |
-            o-improve/refactor  MR !1 (some_other_user) rebase=no push=no
-              |
-              o-comments/add_docstrings  MR !2 (some_other_user) rebase=no push=no
-                |
-                o-sphinx_export *
-            """
-        )
-
     @staticmethod
     def gitlab_api_state_for_test_gitlab_checkout_mrs_from_fork_with_deleted_repo() -> MockGitLabAPIState:
-        return MockGitLabAPIState(
+        return MockGitLabAPIState.with_mrs(
             mock_mr_json(head='feature/allow_checkout', base='develop', number=2, repo_id=0, state='closed'),
             mock_mr_json(head='bugfix/allow_checkout', base='develop', number=3)
         )
 
     def test_gitlab_checkout_mrs_from_fork_with_deleted_repo(self, mocker: MockerFixture) -> None:
         self.patch_symbol(mocker, 'git_machete.code_hosting.OrganizationAndRepository.from_url', mock_from_url)
+        self.patch_symbol(mocker, 'git_machete.gitlab.GitLabToken.for_domain', mock_gitlab_token_for_domain_fake)
         self.patch_symbol(mocker, 'urllib.request.urlopen',
                           mock_urlopen(self.gitlab_api_state_for_test_gitlab_checkout_mrs_from_fork_with_deleted_repo()))
 
-        (
-            self.repo_sandbox.new_branch("root")
-            .commit('initial master commit')
-            .push()
-            .new_branch('develop')
-            .commit('initial develop commit')
-            .push()
-        )
+        (local_path, remote_path) = create_repo_with_remote()
+        new_branch("root")
+        commit('initial master commit')
+        push()
+        new_branch('develop')
+        commit('initial develop commit')
+        push()
 
-        self.repo_sandbox \
-            .chdir(self.repo_sandbox.remote_path)\
-            .execute("git branch merge-requests/2/head develop")\
-            .chdir(self.repo_sandbox.local_path)
+        os.chdir(remote_path)
+        execute("git branch merge-requests/2/head develop")
+        os.chdir(local_path)
 
         body: str = \
             """
@@ -432,7 +332,7 @@ class TestGitLabCheckoutMRs(BaseTest):
 
     @staticmethod
     def gitlab_api_state_for_test_gitlab_checkout_mrs_of_current_user_and_other_users() -> MockGitLabAPIState:
-        return MockGitLabAPIState(
+        return MockGitLabAPIState.with_mrs(
             mock_mr_json(head='chore/redundant_checks', base='restrict_access', number=18),
             mock_mr_json(head='restrict_access', base='allow-ownership-link', number=17, user='gitlab_user'),
             mock_mr_json(head='allow-ownership-link', base='bugfix/feature', number=12),
@@ -450,59 +350,59 @@ class TestGitLabCheckoutMRs(BaseTest):
         self.patch_symbol(mocker, 'urllib.request.urlopen',
                           mock_urlopen(self.gitlab_api_state_for_test_gitlab_checkout_mrs_of_current_user_and_other_users()))
 
-        (
-            self.repo_sandbox.new_branch("root")
-            .commit("initial commit")
-            .new_branch("develop")
-            .commit("first commit")
-            .push()
-            .new_branch("enhance/feature")
-            .commit("introduce feature")
-            .push()
-            .new_branch("bugfix/feature")
-            .commit("bugs removed")
-            .push()
-            .new_branch("allow-ownership-link")
-            .commit("fixes")
-            .push()
-            .new_branch('restrict_access')
-            .commit('authorized users only')
-            .push()
-            .new_branch("chore/redundant_checks")
-            .commit('remove some checks')
-            .push()
-            .check_out("root")
-            .new_branch("master")
-            .commit("Master commit")
-            .push()
-            .new_branch("hotfix/add-trigger")
-            .commit("HOTFIX Add the trigger")
-            .push()
-            .new_branch("ignore-trailing")
-            .commit("Ignore trailing data")
-            .push()
-            .delete_branch("root")
-            .new_branch('chore/fields')
-            .commit("remove outdated fields")
-            .push()
-            .check_out('develop')
-            .new_branch('enhance/add_user')
-            .commit('allow externals to add users')
-            .push()
-            .new_branch('bugfix/add_user')
-            .commit('first round of fixes')
-            .push()
-            .new_branch('testing/add_user')
-            .commit('add test set for add_user feature')
-            .push()
-            .new_branch('chore/comments')
-            .commit('code maintenance')
-            .push()
-            .check_out('master')
-        )
+        create_repo_with_remote()
+        new_branch("root")
+        commit("initial commit")
+        new_branch("develop")
+        commit("first commit")
+        push()
+        new_branch("enhance/feature")
+        commit("introduce feature")
+        push()
+        new_branch("bugfix/feature")
+        commit("bugs removed")
+        push()
+        new_branch("allow-ownership-link")
+        commit("fixes")
+        push()
+        new_branch('restrict_access')
+        commit('authorized users only')
+        push()
+        new_branch("chore/redundant_checks")
+        commit('remove some checks')
+        push()
+        check_out("root")
+        new_branch("master")
+        commit("Master commit")
+        push()
+        new_branch("hotfix/add-trigger")
+        commit("HOTFIX Add the trigger")
+        push()
+        new_branch("ignore-trailing")
+        commit("Ignore trailing data")
+        push()
+        delete_branch("root")
+        new_branch('chore/fields')
+        commit("remove outdated fields")
+        push()
+        check_out('develop')
+        new_branch('enhance/add_user')
+        commit('allow externals to add users')
+        push()
+        new_branch('bugfix/add_user')
+        commit('first round of fixes')
+        push()
+        new_branch('testing/add_user')
+        commit('add test set for add_user feature')
+        push()
+        new_branch('chore/comments')
+        commit('code maintenance')
+        push()
+        check_out('master')
+
         for branch in ('chore/redundant_checks', 'restrict_access', 'allow-ownership-link', 'bugfix/feature', 'enhance/add_user',
                        'testing/add_user', 'chore/comments', 'bugfix/add_user'):
-            self.repo_sandbox.delete_branch(branch)
+            delete_branch(branch)
 
         body: str = \
             """
@@ -512,13 +412,6 @@ class TestGitLabCheckoutMRs(BaseTest):
                         chore/fields
             develop
                 enhance/feature
-                    bugfix/feature
-                        allow-ownership-link
-                            restrict_access
-                                chore/redundant_checks
-            bugfix/add_user
-                testing/add_user
-                    chore/comments
             """
         rewrite_branch_layout_file(body)
 
@@ -596,9 +489,10 @@ class TestGitLabCheckoutMRs(BaseTest):
 
     def test_gitlab_checkout_mrs_misc_failures_and_warns(self, mocker: MockerFixture) -> None:
         self.patch_symbol(mocker, 'git_machete.code_hosting.OrganizationAndRepository.from_url', mock_from_url)
-        self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(MockGitLabAPIState()))
+        self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(MockGitLabAPIState.with_mrs()))
 
         self.patch_symbol(mocker, 'git_machete.gitlab.GitLabToken.for_domain', mock_gitlab_token_for_domain_none)
+        create_repo_with_remote()
         assert_success(
             ["gitlab", "checkout-mrs", "--all"],
             """
@@ -630,13 +524,13 @@ class TestGitLabCheckoutMRs(BaseTest):
             ["gitlab", "checkout-mrs", "--mine"],
             """
             Checking for open GitLab MRs... OK
-            Warn: Current user gitlab_user has no open merge request in project example-org/example-repo
+            Warn: User gitlab_user has no open merge request in project example-org/example-repo
             """
         )
 
     @staticmethod
     def gitlab_api_state_with_mr_cycle() -> MockGitLabAPIState:
-        return MockGitLabAPIState(
+        return MockGitLabAPIState.with_mrs(
             mock_mr_json(head='bugfix/feature', base='chore/redundant_checks', number=6),
             mock_mr_json(head='chore/redundant_checks', base='restrict_access', number=18),
             mock_mr_json(head='restrict_access', base='allow-ownership-link', number=17),
@@ -645,23 +539,22 @@ class TestGitLabCheckoutMRs(BaseTest):
 
     def test_gitlab_checkout_mrs_forming_a_cycle(self, mocker: MockerFixture) -> None:
         self.patch_symbol(mocker, 'git_machete.code_hosting.OrganizationAndRepository.from_url', mock_from_url)
+        self.patch_symbol(mocker, 'git_machete.gitlab.GitLabToken.for_domain', mock_gitlab_token_for_domain_fake)
         self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(self.gitlab_api_state_with_mr_cycle()))
 
-        (
-            self.repo_sandbox
-            .new_branch("bugfix/feature")
-            .commit("bugs removed")
-            .push()
-            .new_branch("allow-ownership-link")
-            .commit("fixes")
-            .push()
-            .new_branch('restrict_access')
-            .commit('authorized users only')
-            .push()
-            .new_branch("chore/redundant_checks")
-            .commit('remove some checks')
-            .push()
-        )
+        create_repo_with_remote()
+        new_branch("bugfix/feature")
+        commit("bugs removed")
+        push()
+        new_branch("allow-ownership-link")
+        commit("fixes")
+        push()
+        new_branch('restrict_access')
+        commit('authorized users only')
+        push()
+        new_branch("chore/redundant_checks")
+        commit('remove some checks')
+        push()
 
         assert_failure(
             ['gitlab', 'checkout-mrs', '--all'],
@@ -671,48 +564,41 @@ class TestGitLabCheckoutMRs(BaseTest):
 
     @staticmethod
     def gitlab_api_state_for_test_gitlab_checkout_mrs_single_mr() -> MockGitLabAPIState:
-        return MockGitLabAPIState(
+        return MockGitLabAPIState.with_mrs(
             mock_mr_json(head='develop', base='master', number=18)
         )
 
     def test_gitlab_checkout_mrs_remote_already_added(self, mocker: MockerFixture) -> None:
         self.patch_symbol(mocker, 'git_machete.git_operations.GitContext.fetch_remote', lambda _self, _remote: None)
-        self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(
-            self.gitlab_api_state_for_test_gitlab_checkout_mrs_single_mr()))
-        (
-            self.repo_sandbox
-            .remove_remote("origin")
-            .add_remote("origin-1", mock_projects()[1]['http_url_to_repo'])
-        )
-        assert_failure(
-            ["gitlab", "checkout-mrs", "--all"],
-            "Could not check out MR !18 because branch develop is already deleted from origin-1."
-        )
+        self.patch_symbol(mocker, 'git_machete.gitlab.GitLabToken.for_domain', mock_gitlab_token_for_domain_fake)
+        gitlab_api_state = self.gitlab_api_state_for_test_gitlab_checkout_mrs_single_mr()
+        self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(gitlab_api_state))
+        create_repo()
+        add_remote("origin-1", gitlab_api_state.projects[1]['http_url_to_repo'])
 
-        (
-            self.repo_sandbox
-            .remove_remote("origin-1")
-            .add_remote("tester", 'https://gitlab.com/tester/lolxd.git')
-        )
         assert_failure(
             ["gitlab", "checkout-mrs", "--all"],
-            "Could not check out MR !18 because branch develop is already deleted from tester/tester."
+            "Could not check out MR !18 because branch develop is already deleted from tester/tester/repo_sandbox."
         )
 
     def test_gitlab_checkout_mrs_org_and_repo_from_config(self, mocker: MockerFixture) -> None:
         self.patch_symbol(mocker, 'git_machete.git_operations.GitContext.fetch_remote', lambda _self, _remote: None)
+        self.patch_symbol(mocker, 'git_machete.gitlab.GitLabToken.for_domain', mock_gitlab_token_for_domain_fake)
         self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(
             self.gitlab_api_state_for_test_gitlab_checkout_mrs_single_mr()))
 
-        (
-            self.repo_sandbox
-            .new_branch("master").commit()
-            .new_branch("develop").commit().push()
-        )
+        create_repo_with_remote()
 
-        self.repo_sandbox.set_remote_url("origin", "https://gitlab.com/example-org/example-repo.git")
-        self.repo_sandbox.set_git_config_key('machete.gitlab.organization', "example-org")
-        self.repo_sandbox.set_git_config_key('machete.gitlab.repository', "example-repo")
+        new_branch("master")
+        commit()
+
+        new_branch("develop")
+        commit()
+        push()
+
+        set_remote_url("origin", "https://gitlab.com/example-org/example-repo.git")
+        set_git_config_key('machete.gitlab.organization', "example-org")
+        set_git_config_key('machete.gitlab.repository', "example-repo")
         assert_success(
             ['gitlab', 'checkout-mrs', '--all'],
             'Checking for open GitLab MRs... OK\n'
@@ -721,19 +607,56 @@ class TestGitLabCheckoutMRs(BaseTest):
 
     def test_gitlab_checkout_mrs_remote_from_config(self, mocker: MockerFixture) -> None:
         self.patch_symbol(mocker, 'git_machete.git_operations.GitContext.fetch_remote', lambda _self, _remote: None)
+        self.patch_symbol(mocker, 'git_machete.gitlab.GitLabToken.for_domain', mock_gitlab_token_for_domain_fake)
         self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(
             self.gitlab_api_state_for_test_gitlab_checkout_mrs_single_mr()))
 
-        (
-            self.repo_sandbox
-            .new_branch("master").commit()
-            .new_branch("develop").commit().push()
-        )
+        create_repo_with_remote()
 
-        self.repo_sandbox.set_remote_url("origin", "https://gitlab.com/example-org/example-repo.git")
-        self.repo_sandbox.set_git_config_key('machete.gitlab.remote', "origin")
+        new_branch("master")
+        commit()
+
+        new_branch("develop")
+        commit()
+        push()
+
+        set_remote_url("origin", "https://gitlab.com/example-org/example-repo.git")
+        set_git_config_key('machete.gitlab.remote', "origin")
         assert_success(
             ['gitlab', 'checkout-mrs', '--all'],
             'Checking for open GitLab MRs... OK\n'
             'MR !18 checked out at local branch develop\n'
         )
+
+    @staticmethod
+    def mrs_for_test_checkout_mrs_main_to_main_pr() -> List[Dict[str, Any]]:
+        return [
+            mock_mr_json(head='fix-10341', base='main', number=2),
+            mock_mr_json(head='main', base='main', number=1, repo_id=2)
+        ]
+
+    def test_gitlab_checkout_mrs_main_to_main_mr(self, mocker: MockerFixture) -> None:
+        self.patch_symbol(mocker, 'git_machete.git_operations.GitContext.fetch_remote', lambda _self, _remote: None)
+        self.patch_symbol(mocker, 'git_machete.code_hosting.OrganizationAndRepository.from_url', mock_from_url)
+        self.patch_symbol(mocker, 'git_machete.gitlab.GitLabToken.for_domain', mock_gitlab_token_for_domain_none)
+
+        create_repo_with_remote()
+
+        second_remote_path = create_repo("second-remote", bare=True, switch_dir_to_new_repo=False)
+        gitlab_api_state = MockGitLabAPIState(
+            self.projects_for_test_gitlab_checkout_prs(second_remote_path),
+            *self.mrs_for_test_checkout_mrs_main_to_main_pr())
+        self.patch_symbol(mocker, 'urllib.request.urlopen', mock_urlopen(gitlab_api_state))
+
+        new_branch("main")
+        commit()
+        push()
+        new_branch("fix-10341")
+        commit()
+        push()
+
+        assert_success(
+            ['gitlab', 'checkout-mrs', '2'],
+            'Checking for open GitLab MRs... OK\n'
+            'MR !2 checked out at local branch fix-10341\n'
+        )
