@@ -1,5 +1,4 @@
 import os
-from tempfile import mkdtemp
 
 from pytest_mock import MockerFixture
 
@@ -9,51 +8,58 @@ from .base_test import BaseTest
 from .mockers import (assert_failure, assert_success,
                       fixed_author_and_committer_date_in_past, launch_command,
                       mock_input_returning, mock_input_returning_y,
-                      overridden_environment, rewrite_branch_layout_file)
+                      overridden_environment, rewrite_branch_layout_file,
+                      sleep, write_to_file)
+from .mockers_git_repository import (add_file_and_commit, add_remote,
+                                     amend_commit, check_out, commit,
+                                     create_repo, create_repo_with_remote,
+                                     delete_branch, get_current_branch,
+                                     get_git_version, merge, new_branch, push,
+                                     remove_remote, reset_to,
+                                     set_git_config_key)
 
 
 class TestTraverse(BaseTest):
 
     def setup_standard_tree(self) -> None:
-        (
-            self.repo_sandbox.new_branch("root")
-            .commit("root")
-            .new_branch("develop")
-            .commit("develop commit")
-            .new_branch("allow-ownership-link")
-            .commit("Allow ownership links")
-            .push()
-            .new_branch("build-chain")
-            .commit("Build arbitrarily long chains")
-            .check_out("allow-ownership-link")
-            .commit("1st round of fixes")
-            .check_out("develop")
-            .commit("Other develop commit")
-            .push()
-            .new_branch("call-ws")
-            .commit("Call web service")
-            .commit("1st round of fixes")
-            .push()
-            .new_branch("drop-constraint")
-            .commit("Drop unneeded SQL constraints")
-            .check_out("call-ws")
-            .commit("2nd round of fixes")
-            .check_out("root")
-            .new_branch("master")
-            .commit("Master commit")
-            .push()
-            .new_branch("hotfix/add-trigger")
-            .commit("HOTFIX Add the trigger")
-            .push()
-            .amend_commit("HOTFIX Add the trigger (amended)")
-            .new_branch("ignore-trailing")
-            .commit("Ignore trailing data")
-            .sleep(1)
-            .amend_commit("Ignore trailing data (amended)")
-            .push()
-            .reset_to("ignore-trailing@{1}")  # noqa: FS003
-            .delete_branch("root")
-        )
+        create_repo_with_remote()
+        new_branch("root")
+        commit("root")
+        new_branch("develop")
+        commit("develop commit")
+        new_branch("allow-ownership-link")
+        commit("Allow ownership links")
+        push()
+        new_branch("build-chain")
+        commit("Build arbitrarily long chains")
+        check_out("allow-ownership-link")
+        commit("1st round of fixes")
+        check_out("develop")
+        commit("Other develop commit")
+        push()
+        new_branch("call-ws")
+        commit("Call web service")
+        commit("1st round of fixes")
+        push()
+        new_branch("drop-constraint")
+        commit("Drop unneeded SQL constraints")
+        check_out("call-ws")
+        commit("2nd round of fixes")
+        check_out("root")
+        new_branch("master")
+        commit("Master commit")
+        push()
+        new_branch("hotfix/add-trigger")
+        commit("HOTFIX Add the trigger")
+        push()
+        amend_commit("HOTFIX Add the trigger (amended)")
+        new_branch("ignore-trailing")
+        commit("Ignore trailing data")
+        sleep(1)
+        amend_commit("Ignore trailing data (amended)")
+        push()
+        reset_to("ignore-trailing@{1}")  # noqa: FS003
+        delete_branch("root")
 
         body: str = \
             """
@@ -67,41 +73,19 @@ class TestTraverse(BaseTest):
                     ignore-trailing
             """
         rewrite_branch_layout_file(body)
-        assert_success(
-            ["status"],
-            """
-            develop
-            |
-            x-allow-ownership-link (ahead of origin)
-            | |
-            | x-build-chain (untracked)
-            |
-            o-call-ws (ahead of origin)
-              |
-              x-drop-constraint (untracked)
-
-            master
-            |
-            o-hotfix/add-trigger (diverged from origin)
-              |
-              o-ignore-trailing * (diverged from & older than origin)
-            """
-        )
 
     def test_traverse_slide_out(self, mocker: MockerFixture) -> None:
-        (
-            self.repo_sandbox
-            .remove_remote()
-            .new_branch("master")
-            .commit()
-            .new_branch("develop")
-            .commit()
-            .new_branch("feature")
-            .commit()
-            .check_out("master")
-            .merge("develop")
-            .check_out("develop")
-        )
+        create_repo()
+        new_branch("master")
+        commit()
+        new_branch("develop")
+        commit()
+        new_branch("feature")
+        commit("feature commit")
+        check_out("master")
+        merge("develop")
+        check_out("develop")
+
         body: str = \
             """
             master
@@ -140,12 +124,15 @@ class TestTraverse(BaseTest):
             """
             master
             |
-            | Some commit message.
+            | feature commit
             o-feature
             """
         )
 
-        self.repo_sandbox.check_out("master").merge("feature").check_out("feature")
+        check_out("master")
+        merge("feature")
+        check_out("feature")
+
         self.patch_symbol(mocker, 'builtins.input', mock_input_returning("y"))
         assert_success(
             ["traverse"],
@@ -160,7 +147,7 @@ class TestTraverse(BaseTest):
 
     def test_traverse_no_remotes(self) -> None:
         self.setup_standard_tree()
-        self.repo_sandbox.remove_remote()
+        remove_remote()
 
         launch_command("traverse", "-Wy")
         assert_success(
@@ -229,7 +216,7 @@ class TestTraverse(BaseTest):
 
     def test_traverse_no_push_override(self) -> None:
         self.setup_standard_tree()
-        self.repo_sandbox.check_out("hotfix/add-trigger")
+        check_out("hotfix/add-trigger")
         launch_command("t", "-Wy", "--no-push", "--push", "--start-from=here")
         assert_success(
             ["status", "-l"],
@@ -260,8 +247,8 @@ class TestTraverse(BaseTest):
               o-ignore-trailing
             """,
         )
-        self.repo_sandbox.check_out("ignore-trailing")
-        self.repo_sandbox.set_git_config_key("machete.traverse.push", "false")
+        check_out("ignore-trailing")
+        set_git_config_key("machete.traverse.push", "false")
         launch_command("t", "-Wy")
         assert_success(
             ["status"],
@@ -306,7 +293,7 @@ class TestTraverse(BaseTest):
             """,
         )
 
-        self.repo_sandbox.set_git_config_key("machete.traverse.push", "true")
+        set_git_config_key("machete.traverse.push", "true")
         launch_command("t", "-Wy", "--no-push-untracked")
         assert_success(
             ["status"],
@@ -330,7 +317,12 @@ class TestTraverse(BaseTest):
         )
 
     def test_traverse_ahead_of_remote_responses(self, mocker: MockerFixture) -> None:
-        self.repo_sandbox.new_branch("master").commit().push().commit()
+        create_repo_with_remote()
+        new_branch("master")
+        commit()
+        push()
+        commit()
+
         rewrite_branch_layout_file("master")
 
         self.patch_symbol(mocker, 'builtins.input', mock_input_returning("q"))
@@ -352,9 +344,12 @@ class TestTraverse(BaseTest):
         assert_success(["traverse"], "Push master to origin? (y, N, q, yq)\n")
 
     def test_traverse_behind_remote_responses(self, mocker: MockerFixture) -> None:
-        self.repo_sandbox.new_branch("master")\
-            .commit().commit().push()\
-            .reset_to("HEAD~")
+        create_repo_with_remote()
+        new_branch("master")
+        commit()
+        commit()
+        push()
+        reset_to("HEAD~")
         rewrite_branch_layout_file("master")
 
         self.patch_symbol(mocker, 'builtins.input', mock_input_returning("q"))
@@ -389,7 +384,12 @@ class TestTraverse(BaseTest):
         )
 
     def test_traverse_diverged_from_and_newer_responses(self, mocker: MockerFixture) -> None:
-        self.repo_sandbox.new_branch("master").commit().push().amend_commit("Different commit message")
+        create_repo_with_remote()
+        new_branch("master")
+        commit()
+        push()
+        amend_commit("Different commit message")
+
         rewrite_branch_layout_file("master")
 
         self.patch_symbol(mocker, 'builtins.input', mock_input_returning("q"))
@@ -424,9 +424,13 @@ class TestTraverse(BaseTest):
         )
 
     def test_traverse_diverged_from_and_older_responses(self, mocker: MockerFixture) -> None:
-        self.repo_sandbox.new_branch("master").commit().push()
+        create_repo_with_remote()
+        new_branch("master")
+        commit()
+        push()
+
         with fixed_author_and_committer_date_in_past():
-            self.repo_sandbox.amend_commit()
+            amend_commit()
         rewrite_branch_layout_file("master")
 
         self.patch_symbol(mocker, 'builtins.input', mock_input_returning("q"))
@@ -496,7 +500,7 @@ class TestTraverse(BaseTest):
 
     def test_traverse_push_config_key(self) -> None:
         self.setup_standard_tree()
-        self.repo_sandbox.set_git_config_key('machete.traverse.push', 'false')
+        set_git_config_key('machete.traverse.push', 'false')
         launch_command("traverse", "-Wy")
         assert_success(
             ["status", "-l"],
@@ -529,30 +533,29 @@ class TestTraverse(BaseTest):
         )
 
     def test_traverse_no_push_no_checkout(self) -> None:
-        (
-            self.repo_sandbox.new_branch("root")
-                .commit("root")
-                .new_branch("develop")
-                .commit("develop commit")
-                .commit("Other develop commit")
-                .push()
-                .new_branch("allow-ownership-link")
-                .commit("Allow ownership links")
-                .push()
-                .check_out("allow-ownership-link")
-                .commit("1st round of fixes")
-                .new_branch("build-chain")
-                .commit("Build arbitrarily long chains")
-                .check_out("root")
-                .new_branch("master")
-                .commit("Master commit")
-                .push()
-                .new_branch("hotfix/add-trigger")
-                .commit("HOTFIX Add the trigger")
-                .push()
-                .amend_commit("HOTFIX Add the trigger (amended)")
-                .delete_branch("root")
-        )
+        create_repo_with_remote()
+        new_branch("root")
+        commit("root")
+        new_branch("develop")
+        commit("develop commit")
+        commit("Other develop commit")
+        push()
+        new_branch("allow-ownership-link")
+        commit("Allow ownership links")
+        push()
+        check_out("allow-ownership-link")
+        commit("1st round of fixes")
+        new_branch("build-chain")
+        commit("Build arbitrarily long chains")
+        check_out("root")
+        new_branch("master")
+        commit("Master commit")
+        push()
+        new_branch("hotfix/add-trigger")
+        commit("HOTFIX Add the trigger")
+        push()
+        amend_commit("HOTFIX Add the trigger (amended)")
+        delete_branch("root")
 
         body: str = \
             """
@@ -604,7 +607,7 @@ class TestTraverse(BaseTest):
     def test_traverse_and_squash(self) -> None:
         self.setup_standard_tree()
 
-        self.repo_sandbox.check_out("hotfix/add-trigger")
+        check_out("hotfix/add-trigger")
         launch_command("traverse", "--fetch", "--start-from=root", "--return-to=here", "-y")
         assert_success(
             ["status", "-l"],
@@ -635,7 +638,7 @@ class TestTraverse(BaseTest):
               o-ignore-trailing
             """,
         )
-        assert self.repo_sandbox.get_current_branch() == "hotfix/add-trigger"
+        assert get_current_branch() == "hotfix/add-trigger"
 
         launch_command("traverse", "-wy")
         assert_success(
@@ -701,20 +704,17 @@ class TestTraverse(BaseTest):
         )
 
     def test_traverse_with_merge(self, mocker: MockerFixture) -> None:
-        (
-            self.repo_sandbox
-            .remove_remote()
-            .new_branch("develop")
-            .commit()
-            .new_branch('mars')
-            .commit()
-            .new_branch('snickers')
-            .commit()
-            .check_out('mars')
-            .commit()
-            .check_out('develop')
-            .commit()
-        )
+        create_repo()
+        new_branch("develop")
+        commit()
+        new_branch('mars')
+        commit()
+        new_branch('snickers')
+        commit()
+        check_out('mars')
+        commit()
+        check_out('develop')
+        commit()
 
         body: str = \
             """
@@ -747,19 +747,17 @@ class TestTraverse(BaseTest):
         )
 
     def test_traverse_with_merge_annotation(self, mocker: MockerFixture) -> None:
-        (
-            self.repo_sandbox.remove_remote()
-            .new_branch("develop")
-            .commit()
-            .new_branch("mars")
-            .commit()
-            .new_branch("snickers")
-            .commit()
-            .check_out("mars")
-            .commit()
-            .check_out("develop")
-            .commit()
-        )
+        create_repo()
+        new_branch("develop")
+        commit()
+        new_branch("mars")
+        commit()
+        new_branch("snickers")
+        commit()
+        check_out("mars")
+        commit()
+        check_out("develop")
+        commit()
 
         body: str = """
             develop
@@ -790,6 +788,41 @@ class TestTraverse(BaseTest):
             """,
         )
 
+    def test_traverse_with_merge_annotation_and_yes_option(self) -> None:
+        create_repo()
+        new_branch("develop")
+        commit()
+        new_branch("mars")
+        commit()
+        new_branch("snickers")
+        commit()
+        check_out("mars")
+        commit()
+        check_out("develop")
+        commit()
+
+        body: str = """
+            develop
+                mars update=merge
+                    snickers
+            """
+        rewrite_branch_layout_file(body)
+
+        with overridden_environment(GIT_EDITOR='false'):
+            # --yes should imply --no-edit-merge, if it doesn't then the command will fail due to a non-zero exit code from the editor
+            launch_command("traverse", "--start-from=root", "--yes"),
+
+        assert_success(
+            ["status"],
+            """
+            develop
+            |
+            o-mars  update=merge
+              |
+              o-snickers *
+            """,
+        )
+
     def test_traverse_qualifiers_no_push(self) -> None:
         self.setup_standard_tree()
 
@@ -919,7 +952,8 @@ class TestTraverse(BaseTest):
             \t\tignore-trailing
             """
         rewrite_branch_layout_file(body)
-        self.repo_sandbox.check_out('develop').merge('call-ws')
+        check_out('develop')
+        merge('call-ws')
 
         launch_command("traverse", "-Wy")
         assert_success(
@@ -944,32 +978,36 @@ class TestTraverse(BaseTest):
         )
 
     def test_traverse_no_managed_branches(self) -> None:
+        create_repo()
 
-        expected_error_message = "No branches listed in .git/machete; " \
-                                 "use git machete discover or git machete edit, or edit .git/machete manually."
+        expected_error_message = """
+          No branches listed in .git/machete. Consider one of:
+          * git machete discover
+          * git machete edit or edit .git/machete manually
+          * git machete github checkout-prs --mine
+          * git machete gitlab checkout-mrs --mine"""
         assert_failure(["traverse"], expected_error_message)
 
     def test_traverse_invalid_flag_values(self) -> None:
-        self.setup_standard_tree()
-        assert_failure(["traverse", "--start-from=nowhere"],
-                       "Invalid argument for --start-from. Valid arguments: here|root|first-root.")
         assert_failure(["traverse", "--return-to=dunno-where"],
-                       "Invalid argument for --return-to. Valid arguments: here|nearest-remaining|stay.")
+                       "Invalid value for --return-to flag: dunno-where. Valid values are here, nearest-remaining, stay")
+        assert_failure(["traverse", "--squash-merge-detection=lolxd"],
+                       "Invalid value for --squash-merge-detection flag: lolxd. Valid values are none, simple, exact")
+        assert_failure(["traverse", "--start-from=nowhere"],
+                       "Invalid value for --start-from flag: nowhere. Valid values are here, root, first-root")
 
     def test_traverse_removes_current_directory(self) -> None:
-        (
-            self.repo_sandbox
-            .new_branch("master")
-            .commit()
-            .new_branch("with-directory")
-            .add_file_and_commit(file_path="directory/file.txt")
-            .check_out("master")
-            .new_branch("without-directory")
-            .commit()
-            .check_out("master")
-            .commit()
-            .check_out("with-directory")
-        )
+        (local_path, _) = create_repo_with_remote()
+        new_branch("master")
+        commit()
+        new_branch("with-directory")
+        add_file_and_commit(file_path="directory/file.txt")
+        check_out("master")
+        new_branch("without-directory")
+        commit()
+        check_out("master")
+        commit()
+        check_out("with-directory")
 
         body: str = \
             """
@@ -1006,7 +1044,7 @@ class TestTraverse(BaseTest):
 
             Reached branch without-directory which has no successor; nothing left to update
             """
-        if self.repo_sandbox.get_git_version() >= (2, 35, 0):
+        if get_git_version() >= (2, 35, 0):
             # See https://github.com/git/git/blob/master/Documentation/RelNotes/2.35.0.txt#L81 for the fix
             assert_success(
                 ["traverse", "-y"],
@@ -1017,44 +1055,40 @@ class TestTraverse(BaseTest):
             assert_success(
                 ["traverse", "-y"],
                 common_expected_output +
-                f"Warn: current directory {self.repo_sandbox.local_path}/directory no longer exists, " +
-                f"the nearest existing parent directory is {self.repo_sandbox.local_path}\n"
+                f"Warn: current directory {local_path}/directory no longer exists, " +
+                f"the nearest existing parent directory is {local_path}\n"
             )
             assert os.path.split(os.getcwd())[-1] != "directory"
 
     def test_traverse_reset_keep_failing(self) -> None:
-        (
-            self.repo_sandbox
-            .new_branch("master")
-            .add_file_and_commit(file_path="foo.txt", file_content="1")
-            .sleep(1)
-            .write_to_file(file_path="foo.txt", file_content="2")
-            .amend_commit()
-            .push()
-            .reset_to("HEAD@{1}")  # noqa: FS003
-            .write_to_file(file_path="foo.txt", file_content="3")
-        )
+        create_repo_with_remote()
+        new_branch("master")
+        add_file_and_commit(file_path="foo.txt", file_content="1")
+        sleep(1)
+        write_to_file(file_path="foo.txt", file_content="2")
+        amend_commit()
+        push()
+        reset_to("HEAD@{1}")  # noqa: FS003
+        write_to_file(file_path="foo.txt", file_content="3")
 
         rewrite_branch_layout_file("master")
 
         assert_failure(
             ["traverse", "--fetch", "-y", "--debug"],
             "Cannot perform git reset --keep origin/master. This is most likely caused by local uncommitted changes.",
-            expected_exception=UnderlyingGitException
+            expected_type=UnderlyingGitException
         )
 
     def test_traverse_with_stop_for_edit(self, mocker: MockerFixture) -> None:
 
-        (
-            self.repo_sandbox
-            .remove_remote()
-            .new_branch("branch-0")
-            .commit()
-            .new_branch("branch-1")
-            .commit()
-            .check_out("branch-0")
-            .commit()
-        )
+        create_repo()
+        new_branch("branch-0")
+        commit()
+        new_branch("branch-1")
+        commit()
+        check_out("branch-0")
+        commit()
+
         rewrite_branch_layout_file("branch-0\n\tbranch-1")
 
         self.patch_symbol(mocker, 'builtins.input', mock_input_returning_y)
@@ -1073,17 +1107,16 @@ class TestTraverse(BaseTest):
     def test_reset_to_remote_after_rebase(self) -> None:
         """Very unlikely case; can happen only in case of divergence of clocks between local and remote
         (which is simulated in this test)."""
-        (
-            self.repo_sandbox
-            .new_branch("branch-0")
-            .commit()
-            .push()
-            .new_branch("branch-1")
-            .commit()
-            .push()
-            .check_out("branch-0")
-            .commit()
-        )
+        create_repo_with_remote()
+        new_branch("branch-0")
+        commit()
+        push()
+        new_branch("branch-1")
+        commit()
+        push()
+        check_out("branch-0")
+        commit()
+
         rewrite_branch_layout_file("branch-0\n\tbranch-1")
 
         with fixed_author_and_committer_date_in_past():
@@ -1112,7 +1145,9 @@ class TestTraverse(BaseTest):
             )
 
     def test_traverse_quit_on_pushing_untracked(self, mocker: MockerFixture) -> None:
-        self.repo_sandbox.new_branch("master").commit()
+        create_repo_with_remote()
+        new_branch("master")
+        commit()
         rewrite_branch_layout_file("master")
         self.patch_symbol(mocker, 'builtins.input', mock_input_returning("q"))
         assert_success(
@@ -1121,27 +1156,26 @@ class TestTraverse(BaseTest):
         )
 
     def test_traverse_multiple_remotes(self, mocker: MockerFixture) -> None:
-        origin_1_remote_path = mkdtemp()
-        origin_2_remote_path = mkdtemp()
-        (
-            self.repo_sandbox
-            .remove_remote("origin")
-            .new_repo(origin_1_remote_path, bare=True, switch_dir_to_new_repo=False)
-            .add_remote("origin_1", origin_1_remote_path)
-            .new_repo(origin_2_remote_path, bare=True, switch_dir_to_new_repo=False)
-            .add_remote("origin_2", origin_2_remote_path)
-        )
+        create_repo()
+        origin_1_remote_path = create_repo("remote-1", bare=True, switch_dir_to_new_repo=False)
+        origin_2_remote_path = create_repo("remote-2", bare=True, switch_dir_to_new_repo=False)
+        add_remote("origin-1", origin_1_remote_path)
+        add_remote("origin-2", origin_2_remote_path)
 
-        self.repo_sandbox.new_branch("master").commit()
+        new_branch("master")
+        commit()
         rewrite_branch_layout_file("master")
 
         self.patch_symbol(mocker, 'builtins.input', mock_input_returning("xd"))
         assert_success(
-            ["traverse"],
+            ["traverse", "--fetch"],
             """
+            Fetching origin-1...
+            Fetching origin-2...
+
             Branch master is untracked and there's no origin remote.
-            [1] origin_1
-            [2] origin_2
+            [1] origin-1
+            [2] origin-2
             Select number 1..2 to specify the destination remote repository, or 'n' to skip this branch, or 'q' to quit the traverse:
 
               master * (untracked)
@@ -1151,35 +1185,36 @@ class TestTraverse(BaseTest):
         )
 
         self.patch_symbol(mocker, 'builtins.input', mock_input_returning("1", "o", "2", "yq"))
+        set_git_config_key("machete.traverse.fetch.origin-2", "false")
         assert_success(
-            ["traverse"],
+            ["traverse", "--fetch"],
             """
+            Fetching origin-1...
+
             Branch master is untracked and there's no origin remote.
-            [1] origin_1
-            [2] origin_2
+            [1] origin-1
+            [2] origin-2
             Select number 1..2 to specify the destination remote repository, or 'n' to skip this branch, or 'q' to quit the traverse:
-            Push untracked branch master to origin_1? (y, N, q, yq, o[ther-remote])
-            [1] origin_1
-            [2] origin_2
+            Push untracked branch master to origin-1? (y, N, q, yq, o[ther-remote])
+            [1] origin-1
+            [2] origin-2
             Select number 1..2 to specify the destination remote repository, or 'n' to skip this branch, or 'q' to quit the traverse:
-            Push untracked branch master to origin_2? (y, N, q, yq, o[ther-remote])
+            Push untracked branch master to origin-2? (y, N, q, yq, o[ther-remote])
             """
         )
 
     def test_traverse_yellow_edges(self, mocker: MockerFixture) -> None:
-        (
-            self.repo_sandbox
-            .remove_remote()
-            .new_branch("master")
-            .commit()
-            .new_branch("develop")
-            .commit()
-            .new_branch("feature-1")
-            .commit()
-            .check_out("develop")
-            .new_branch("feature-2")
-            .commit()
-        )
+        create_repo()
+        new_branch("master")
+        commit()
+        new_branch("develop")
+        commit()
+        new_branch("feature-1")
+        commit()
+        check_out("develop")
+        new_branch("feature-2")
+        commit()
+
         body: str = \
             """
             master
